/*   BMPSpeed Benchmark Copyright (C) Roy Longbottom 2004
 *
 *  BMPSpd.exe has two built in 24 bit .BMP files size 204 x 204 and 288 
 *  x 288 pixels or about 0.125 and 0.25 MBytes. The program alternately 
 *  doubles the pixel dimensions to produce files for testing at 0.5, 1, 
 *  2, 4 up to 50% of RAM size (maximum image 512 MB). Extra copies for 
 *  editing result in memory demands of greater than memory capacity, 
 *  leading to paging to/from disk. Five tests are run at each size, run 
 *  times being saved in log file BMPTime.txt.
 *
 *  1 - Enlarge with blur editing and display.
 *  2 - Save enlargement to disk.
 *  3 - Load from disk, format and display.
 *  4 - Copy from memory scrolling.
 *  5 - Make an extra copy rotating 90 degrees and display.
 *
 *  Data transfer speeds in MB/second are also recorded for Test 4 where 
 *  displayed data might be from video RAM cache, main RAM or disk page 
 *  file. 
 *
 *  Modules
 *
 *  config.cpp - via WhatConfig() identifies CPU type/speed, memory
 *          size, OS version, disk space, also start_time(),
 *          end_time(), local_time() for date and clock time.
 *  
 *  BMPSpeed.rc - Menus and Dialog boxes AboutDlgProc(), 
 *                ResultSelect(), ResultData(), TestPanel(),
 *                ProgressProc() and ProgressProc2().
 *
 *  BMPSpeed.h and BMPSpd.ico
 *
 *  B204x204.bmp and B288x288.bmp (not needed at run time)
 *
 *  vesa.exe - needed at run time for graphics information
 *
 *  BMPSpeed.cpp
 * 
 *  Particularly with Windows XP, on running a benchmark that displays
 *  on-going results, the window for the latter can be covered by
 *  another window. To overcome this, all tests are run via the WinMain 
 *  message processing loop using a phase variable and runTests().
 *
 *
 *  WinMain() - usual Windows functions and autoRun
 *  InitApp() - usual Windows functions, processes command line,
 *              allocates variable values, verifies monitor settings.
 *              Batch commands in command line - RUNSECS, MAXSIZE, AUTORUN,
 *              NOVESA, DEBUG, MINTEST (run first few tests), WRITE_THROUGH
 *  WindowProc() - usual Windows functions for msgs, buttons & menus
 *  AboutDlgProc() - displays Help and About
 *  TestPanel() - Main testing window - Information boxes TEST_NOTES,
 *         INIT_SET, INFO, Maximum Image Size drop down selection
 *         MAX_SIZE, WRITE_THROUGH tick box, buttons TEST_IT, EXIT_BUTTON
 *         SEE_RESULTS, FIND_RESULTS
 *
 *  setTests() - via autoRun or TEST_IT button - open log file, allocate
 *         initial memory, load built-in BMPs, CreateDIBitmaps
 *
 *  Menu Selections
 *
 *  MENU_RUN as TEST_IT button, MENU_EXIT as EXIT_BUTTON as , MENU_DISPLAY
 *  as SEE_RESULTS, MENU_SELECT as FIND_RESULTS, MENU_RHELP general help,
 *  MENU_BHELP batch parameters help, MENU_VERS version help
 *
 *  runTests() 
 *
 *  phase 1  - Test 1, select bmp, allocate memory allocate1()
 *  phase 2  - Test 1, enlarge() and release input memory
 *  phase 3  - Test 1, ProgressProc2(), CreateDIB Output
 *  phase 4  - Test 1, displayBMP3(), delete progress display
 *  phase 5  - Test 2, saveBMP() and free memory
 *  phase 6  - Test 3, allocate memory and loadInputBMP(), ProgressProc2()
 *                 CreateDIBitmap (see Note 1), delete progress display
 *  phase 7  - Test 3, ProgressProc2(), displayBMP1(), delete progress display
 *  phase 8  - Test 4, scroll()
 *  phase 9  - Test 5, allocate memory allocate1()
 *  phase 10 - Test 5, rotate()
 *  phase 11 - Test 5, displayBMP2()
 *  phase 12 - displayBMP1(), save results
 *  phase 13 - endPrint()
 *
 *  allocate1() - ProgressProc2(), allocate memory, delete progress display
 *  enlarge() - startProgress(), enlarge2(), endProgress()
 *  enlarge2() - moving RGB bytes with calculations
 *  displayBMP1() - copy data or part to screen - see Note 1
 *  displayBMP2() - CreateDIBitmap, copy data or part to screen - see Note 1
 *                  delete a bitmap and progress display     
 *  displayBMP3() - copy data or part to screen - see Note 1
 *  saveBMP() - startProgress(), write file, endProgress()
 *  loadInputBMP() - startProgress(), read file, endProgress()
 *  scroll() - calibrate increment, move1() or move2()
 *  move1() - copy data or part to screen - see Note 1, down
 *  move2() - copy data or part to screen - see Note 1, up
 *  rotate() - startProgress(), rotate by copying 3 bytes at a time
 *             endProgress()
 *  startProgress() - ProgressProc(), Create Child Window for progress bar,
 *                    set range. See PBM_SETSTEP, PBM_STEPIT
 *  endProgress() - delete progress bar window and dialog box
 *  ProgressProc() - Dialog box with progressText message in TEST_BOX
 *                   displayed where function is in steps suitable for
 *                   progress bar
 *  ProgressProc2() - Dialog box with progressText message in NOTE_BOX
 *                    displayed when waiting for something to happen
 *  finish() - close log file if open, addVesa(), delete/free objects
 *  initFail() - save error messages to log file, endPrint(), finish()
 *  endPrint() - save configuration and memory statistics to log file
 *               close log file
 *  GetMemoryStatus() - get details of free and used memory
 *  ResultSelect() - Displays list of results in log file and selection
 *  ResultData() - Displays selected results
 *  displayWhatever() - selects 1 of above 3 dialog boxes 
 *  findLastLog() - first reads log file to count number of entries
 *                  then runs readData() - Benchmark specific
 *  readData() - reads configuration data for all entries or a set of
 *               results - Benchmark specific 
 *  GetFileName() - calls Windows standard Open File dialog
 *  addVesa() - add some details of graphics used to log file
 *
 *  Note 1
 *
 *  If CreateDIBitmap is successful BitBlt can be used. If not,
 *  the slower StretchDIBits has to be used. This depends on which
 *  version of Windows and video RAM size
 * 
 *   ************************* WARNING ****************************** 
 *  comctl32.lib MUST BE INCLUDED in compiler Link Library modules.
 *
 */
 
#define NAME "BMPSpeed"
#define TITLE " BMP Save/Load/Move/Edit Benchmark "
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <commctrl.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>

#include <time.h>
#include "bmpspeed.h"
#include "asmtime.h"


//  ************** PROTOPTYPES *****************

BOOL FAR PASCAL AboutDlgProc( HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL TestPanel( HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL ResultData( HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL ResultSelect( HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL ProgressProc( HWND, UINT, WPARAM, LPARAM);
BOOL FAR PASCAL ProgressProc2( HWND, UINT, WPARAM, LPARAM);
void local_time();
VOID GetMemoryStatus();
static BOOL GetFileName();
BOOL setTests();
BOOL runTests();
void displayWhatever(HWND);
BOOL initFail();
BOOL finish();
BOOL findLastLog();
BOOL readData(int resCount2);
BOOL addVesa();
BOOL loadInputBMP();
BOOL displayBMP1(HDC);
BOOL displayBMP2();
BOOL displayBMP3(HDC);
BOOL saveBMP();
BOOL enlarge2();
BOOL enlarge();
BOOL allocate1();
BOOL scroll();
BOOL rotate();
BOOL startProgress();
BOOL endProgress();
BOOL move1(int, HDC);
BOOL move2(int, int, HDC);
VOID endPrint();

#define version "BMP Benchmark Version 2.2x for 64 bit Windows"
// #define version "BMP Benchmark Version 2.2x for 32 bit Windows"

DWORD   ramMB;
DWORD   percentMB;
DWORD   ramFree;
DWORD   pageMB;
DWORD   pageFree;
DWORD64 virtMB;
DWORD64 virtFree;

DWORD   percentUse;
DWORD   minFreeRam;
DWORD   minFreePage;
DWORD64 minFreeVirt;

HINSTANCE          MyInstance;
TEXTMETRIC         tmt;
FILE               *outfile = NULL;
RECT               clientRect;
RECT               rcClient;
HWND               mainHWND;
HWND               testHwnd = NULL;
HWND               resultHwnd = NULL;
HWND               selectHwnd = NULL;
HWND               progressHwnd = NULL;
LPBITMAPINFO       lpInputDIB1 = NULL;
LPBITMAPINFO       lpInputDIB2 = NULL;
LPBITMAPINFO       lpInputDIBX = NULL;
LPBITMAPINFO       lpOutputDIB = NULL;
BITMAPFILEHEADER   bfh;
LPBITMAPINFOHEADER lpbi;
LPBITMAPINFOHEADER lpbi2;
HDC                hdcLoad1 = NULL;
HDC                hdcLoad2 = NULL;
HDC                hdcOut = NULL;
HDC                hdcLoadX = NULL;
HBITMAP            hLoadBitmap1 = NULL;
HBITMAP            hLoadBitmap2 = NULL;
HBITMAP            hLoadBitmapX = NULL;
HBITMAP            hOutBitmap = NULL;
LPSTR              pBuf;
LPSTR              pBuf2;
HINSTANCE          g_hinst;
HWND               hwndPB;
MEMORYSTATUSEX       lpBuffer;
HCURSOR            hcur;

int     screenWide;
int     screenHigh;
int     bitsaPixel;
int     imageWide;
int     imageHigh;
WORD     imagebpp;
int     freqStart;
int     keyPress;
int     runTime = 2;
int     resCount;
int     resultsIn;
int     wideUsed[40];
int     highUsed[40];
int     bitsUsed[40];
int     sizeToRead;
int     dataLength;
int     sizeToSave;
int     fileNumber;
int     imageDim[20];
int     fileSize[20];
int     fileXtra = 122880;
int     filePasses[20];
int     lastFile; // = 12
int     idown = 0;
int     iright = 0;
int     doWide;
int     doHigh;
int     maxSize;
int     pbcount;
int     maxImage;
int     maxMB = 512;
int     passCount;
int     gotPixel;
int     phase = 0;
int     useBlt;


double  results[40][10];

BOOL    escPress = FALSE;
BOOL    autoRun = FALSE;
BOOL    displayToDo = TRUE;
BOOL    doVesa = FALSE;
BOOL    useLast = TRUE;
BOOL    useTestPanel = TRUE;
BOOL    showIndex = FALSE;
BOOL    runDone = FALSE;
BOOL    testsDone = FALSE;
BOOL    firstOf2;
BOOL    doBlt = TRUE;
BOOL    doBlt2 = TRUE;
BOOL    bQuit = FALSE;
BOOL    debug = FALSE;
BOOL    mintest = FALSE;
BOOL    canPaint = FALSE;
BOOL    displayOut1 = FALSE;
BOOL    writeThru = FALSE;

char   heading1[11][8];
char   heading2[11][8];
char   failMessage[200];
char   testTitle[11][40] = {" "};
char   logFile[100] = "BMPTime.txt";
char   commandLine[256] = "";
char   note1[100] = " Results are saved in log file BMPTime.txt";
char   note2[600] = " Commands ";
char   note3[100] = " ";
char   note4[100] = " ";
char   meminfo1[8][100];
char   meminfo2[8][100];
char   meminfo3[10][100];
char   diskinfo[100];
char   startSettings[100];
char   currentDir[MAX_PATH];
char   searchDir[MAX_PATH];
char   thisLog[256];
char   config[4][100];
char   details[50][200];
char   BMPfile[20][20];
char   fileToLoad[20];
char   fileToSave[20] = "SaveIt.bmp";
char   *input1;
char   *input2;
char   *inputX;
char   *output1;
char   *dataAddress;
char   windowText[200];
char   progressText[100];
char    timeday[30];
double freeDiskMB;
double totalDiskMB;

// updateFrame

BOOL updateFrame()
{
    return TRUE;
                     
}  //  updateFrame




// WindowProc - handle messages for the main application window

LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{
    HDC hdc;
    PAINTSTRUCT      ps;
      
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
    
    case WM_CREATE:
        break;

    case WM_DESTROY:
        finish();
        bQuit = TRUE;  
        break;

    case WM_COMMAND:
        switch( LOWORD( wparam ) )
        {            
        case MENU_ABOUT:
            if (testHwnd)   DestroyWindow(testHwnd);
            DialogBox( MyInstance,"AboutBox", hwnd, (DLGPROC)AboutDlgProc );
            displayWhatever(hwnd);
            break;
                      
        case MENU_RUN:
            if (testHwnd)   DestroyWindow(testHwnd);
            hcur = SetCursor(LoadCursor(NULL, IDC_WAIT));
            if (!setTests())
            {
                SetCursor(hcur);
                return FALSE;
            }
            SetCursor(hcur);
            break;

        case MENU_EXIT:
            runDone = FALSE;
            finish();
            bQuit = TRUE;  
            break;    

        case MENU_DISPLAY:
            strcpy(thisLog, logFile);
            SetCurrentDirectory(currentDir);
            useTestPanel = FALSE;
            useLast = TRUE;
            showIndex = FALSE;
            if (!findLastLog()) useTestPanel = TRUE;
            displayWhatever(mainHWND);                                      
            break;

        case MENU_SELECT:
            strcpy(thisLog, logFile);
            SetCurrentDirectory(searchDir);
            if  (GetFileName())
            {                
                GetCurrentDirectory(MAX_PATH, searchDir);
                useTestPanel = FALSE;
                showIndex = TRUE;
                useLast = FALSE;
                if (!findLastLog()) useTestPanel = TRUE;
            }
            SetCurrentDirectory(currentDir);
            displayWhatever(mainHWND);                                      
            break;

        case MENU_RHELP:
            if (testHwnd)   DestroyWindow(testHwnd);
            DialogBox( MyInstance,"RelHelp", hwnd, (DLGPROC)AboutDlgProc );
            displayWhatever(mainHWND);                                      
            break;

        case MENU_BHELP:
            if (testHwnd)   DestroyWindow(testHwnd);
            DialogBox( MyInstance,"BatchHelp", hwnd, (DLGPROC)AboutDlgProc );
            displayWhatever(mainHWND);                                      
            break;

        case MENU_VERS:
            if (testHwnd)   DestroyWindow(testHwnd);
            DialogBox( MyInstance,"VersionInfo", hwnd, (DLGPROC)AboutDlgProc );
            displayWhatever(mainHWND);                                      
            break;
        }

    case WM_PAINT:
        hdc = BeginPaint(hwnd, &ps);
        GetClientRect(mainHWND, &clientRect);
        Rectangle(hdc, clientRect.left, clientRect.top, clientRect.right, clientRect.bottom);
        if(runDone)
        {
            hcur = SetCursor(LoadCursor(NULL, IDC_WAIT));
            if (debug)
            {
                fprintf (outfile, " Phase %2d Size %d WM_PAINT, ", phase, fileSize[fileNumber]);
                GetMemoryStatus();
                fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
                fflush (outfile);
            }
            if (displayOut1)
            {
                if (doBlt2)
                {
                    BitBlt(hdc, 0, 0, doWide, doHigh, hdcOut, 0, 0, SRCCOPY);
                }
                else
                {
                    StretchDIBits(hdc, 0, 0, doWide, doHigh,
                                       0, 0, doWide,  doHigh, output1+54,
                                       (LPBITMAPINFO)lpbi2, DIB_RGB_COLORS, SRCCOPY); 
                }
            }
            else if (canPaint)
            {
                if (doBlt)
                {
                     BitBlt(hdc, 0, 0, doWide, doHigh, hdcLoadX, iright, idown, SRCCOPY);
                }
                else
                {        
                    StretchDIBits(hdc, 0, 0, doWide, doHigh,
                                  iright, 0, doWide,  doHigh, pBuf,
                           (LPBITMAPINFO)lpbi, DIB_RGB_COLORS, SRCCOPY);
                }
            }
            SetCursor(hcur);           
        }
        EndPaint(hwnd, &ps);
        break;

    case WM_ACTIVATE:
         break;
    } 
    return DefWindowProc( hwnd, msg, wparam, lparam );

} // WindowProc





// This function is called if the initialization function fails

BOOL initFail()
{
   canPaint = FALSE;
   if (outfile != NULL)
   {
       fprintf (outfile, "\n  ERROR MESSAGE %s \n\n", failMessage);
       endPrint();
   }
   if (!autoRun) MessageBox( mainHWND, failMessage, "See Log File", MB_OK );
   finish();
   bQuit = TRUE;  
   return FALSE;

} // initFail

BOOL finish()
{
   
   if (outfile != NULL)
   {
       fflush (outfile);
       fclose(outfile);
   }
   if (testsDone && doVesa) addVesa();
   runDone = FALSE;
   for (fileNumber=2; fileNumber<lastFile; fileNumber++)
   {
         DeleteFile(BMPfile[fileNumber]);
   }

   if(hwndPB) DestroyWindow(hwndPB); 
   if (progressHwnd) DestroyWindow(progressHwnd);
   canPaint = FALSE;
   if(hdcLoad1) DeleteDC( hdcLoad1 );
   if(hLoadBitmap1) DeleteObject( hLoadBitmap1 );
   if(hdcLoad2) DeleteDC( hdcLoad2 );
   if(hLoadBitmap2) DeleteObject( hLoadBitmap2 );
   if(hdcOut) DeleteDC( hdcOut );
   if(hOutBitmap) DeleteObject(hOutBitmap);

   if (input1)  VirtualFree(input1, 0, MEM_RELEASE);
   if (input2)  VirtualFree(input2, 0, MEM_RELEASE);
   if (output1) VirtualFree(output1, 0, MEM_RELEASE);
   return FALSE;

} // finish

// InitApp - register window class for the application,
//                 and do any other application initialization

static BOOL InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow )
{
    HWND                hwnd;
    WNDCLASS            wc;
    HDC hdc;
    int j, m;
       
    cmdshow = cmdshow;
    
    
    // set up and register window class

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof( DWORD );
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon( hInstance, "GenericIcon" );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = (HBRUSH)GetStockObject( BLACK_BRUSH );
    wc.lpszMenuName = "GenericMenu";
    wc.lpszClassName = NAME;
    RegisterClass(&wc);

    
     // create main window

    hwnd = CreateWindowEx(
        0,
        NAME,  
        TITLE,  
        WS_POPUP  | WS_MAXIMIZE | WS_CAPTION | WS_SYSMENU,
        0,
        0,
        GetSystemMetrics(SM_CXSCREEN),
        GetSystemMetrics(SM_CYSCREEN),
        NULL,
        NULL,
        hInstance,
        NULL );

                    
    if( !hwnd ) return( FALSE );

    mainHWND = hwnd;

    strcpy(commandLine, strupr(cmdline));
    if (strlen(commandLine) > 1)
    {
        strcat(note2, commandLine);
    }
    else
    {
        strcpy(note2, "");
    }
    GetCurrentDirectory(MAX_PATH, currentDir);
    strcpy(searchDir, currentDir);
 
     // set up data associated with this window

    hdc = GetDC(hwnd);
    bitsaPixel = GetDeviceCaps(hdc, BITSPIXEL);
    screenWide = GetDeviceCaps(hdc, HORZRES);    
    screenHigh = GetDeviceCaps(hdc, VERTRES);
    freqStart = GetDeviceCaps(hdc, VREFRESH);
    ReleaseDC(hwnd, hdc);

    strcpy(heading1[0], "bltfast");
    strcpy(heading2[0], "vid_ram");
    strcpy(heading1[1], "blt_and");
    strcpy(heading2[1], "stretch");
    strcpy(heading1[2], "bltfill");
    strcpy(heading2[2], "rectngl");
    strcpy(heading1[3], "bltrect");
    strcpy(heading2[3], "+vsynch");
    strcpy(heading1[4], "bltfast");
    strcpy(heading2[4], "cpu_ram");
    strcpy(heading1[5], "+vsynch");
    strcpy(heading2[5], "    FPS");
    strcpy(heading1[6], "vid_ram");
    strcpy(heading2[6], "    FPS");
    strcpy(heading1[7], "cpu_ram");
    strcpy(heading2[7], "    FPS");

    strcpy(BMPfile[0],  "B204x204.bmp");
    strcpy(BMPfile[1],  "B288x288.bmp");
    strcpy(BMPfile[2],  "B408x408.bmp");
    strcpy(BMPfile[3],  "B576x576.bmp");
    strcpy(BMPfile[4],  "B816x816.bmp");
    strcpy(BMPfile[5],  "B1k1x1k1.bmp");
    strcpy(BMPfile[6],  "B1k6x1k6.bmp");
    strcpy(BMPfile[7],  "B2k3x2k3.bmp");
    strcpy(BMPfile[8],  "B3k2x3k2.bmp");
    strcpy(BMPfile[9],  "B4k6x4k6.bmp");
    strcpy(BMPfile[10], "B6k5x6k5.bmp");
    strcpy(BMPfile[11], "B9k2x9k2.bmp");
    strcpy(BMPfile[12], "B13kx13k.bmp");

    imageDim[0]  = 204;   //  0.1 MB
    imageDim[1]  = 288;   //  0.2 MB
    imageDim[2]  = 408;   //  0.5 MB
    imageDim[3]  = 576;   //  1.0 MB
    imageDim[4]  = 816;   //  2.0 MB
    imageDim[5]  = 1152;  //  4.0 MB
    imageDim[6]  = 1632;  //  8.0 MB
    imageDim[7]  = 2304;  //  16  MB
    imageDim[8]  = 3264;  //  32  MB
    imageDim[9]  = 4608;  //  64  MB
    imageDim[10] = 6528;  //  128 MB
    imageDim[11] = 9216;  //  255 MB
    imageDim[12] = 13056; //  510 MB

    fileSize[0]  = 126976;
    fileSize[1]  = 249856;
    fileSize[2]  = 499712;
    fileSize[3]  = 999424;
    fileSize[4]  = 2097152;
    fileSize[5]  = 4194304;
    fileSize[6]  = 8388608;
    fileSize[7]  = 16777216;
    fileSize[8]  = 33554432;
    fileSize[9]  = 67108864;
    fileSize[10] = 134217728;
    fileSize[11] = 268435456;
    fileSize[12] = 536870912;
        
    filePasses[0] = 1;
    filePasses[1] = 1;
    filePasses[2] = 1;
    filePasses[3] = 1;
    filePasses[4] = 2;
    filePasses[5] = 4;
    filePasses[6] = 8;
    filePasses[7] = 16;
    filePasses[8] = 32;
    filePasses[9] = 64;
    filePasses[10] = 128;
    filePasses[11] = 256;
    filePasses[12] = 512;

    lastFile = 13;                                // max 512 MB
    getDetails();
//    sprintf (diskinfo, "  Free disk space MB %c%d", 9, (int)freeDiskMB);
    if (ramsize <  800) lastFile = 12; // max 256 MB
    if (ramsize <  400) lastFile = 11; // max 128 MB
    if (ramsize <  200) lastFile = 10; // max  64 MB
    if (ramsize <  100) lastFile =  9; // max  32 MB
    if (ramsize <   50) lastFile =  8; // max  16 MB
    if (ramsize <   25) lastFile =  7; // max   8 MB
    if (ramsize <   12) lastFile =  6; // max   4 MB
    if (ramsize <    6) lastFile =  5; // max   2 MB
    if (ramsize <    3)
    {
        sprintf(failMessage, "Insufficient memory");
        return initFail();
    }

    lpBuffer.dwLength = sizeof(lpBuffer);

    keyPress = GetAsyncKeyState(VK_ESCAPE);
    if (freqStart > 1)
    {
        sprintf(startSettings, " Screen setting %d x %d x %d, at %d Hz",
                  screenWide, screenHigh, bitsaPixel, freqStart);
    }
    else
    {
        sprintf(startSettings, " Initial screen setting %d x %d x %d, at default Hz",
                  screenWide, screenHigh, bitsaPixel);
    }

    maxImage = lastFile;
     
    // command line scan
    if (strlen(commandLine))
    {
       for (j=0; j<lstrlen(commandLine); j++)
       {
          if (strnicmp(&commandLine[j],  "RUNSECS ", 8) == 0)
          {
              sscanf(&commandLine[j+8], "%d", &runTime);
              if (runTime < 1) runTime = 1;
              if (runTime > 900) runTime = 900;
          }
          if (strnicmp(&commandLine[j],  "MAXSIZE ", 8) == 0)
          {
              sscanf(&commandLine[j+8], "%d", &m);
              if (m < 1) m = 1;
              if (m > maxMB) m = maxMB;
              if (m == 512)  maxImage = 13;
              if (m < 512)   maxImage = 12;
              if (m < 256)   maxImage = 11;
              if (m < 128)   maxImage = 10;
              if (m < 64)    maxImage =  9;
              if (m < 32)    maxImage =  8;
              if (m < 16)    maxImage =  7;
              if (m <  8)    maxImage =  6;
              if (m <  4)    maxImage =  5;
              if (m <  2)    maxImage =  4;
          }

          if (strnicmp(&commandLine[j],  "AUT", 3) == 0)
          {
              autoRun = TRUE;
          }
          if (strnicmp(&commandLine[j],  "NOV", 3) == 0)
          {
              doVesa = FALSE;
          }
          if (strnicmp(&commandLine[j],  "DEB", 3) == 0)
          {
              debug = TRUE; 
          }
          if (strnicmp(&commandLine[j],  "MINT", 4) == 0)
          {
              mintest = TRUE; 
          }
          if (strnicmp(&commandLine[j],  "WRIT", 4) == 0)
          {
              writeThru = TRUE; 
          }
       }
     
    }

    lastFile = maxImage;
    maxSize = fileSize[lastFile - 1] / 1048576;

    strcpy(testTitle[0], "BltFast VRAM ");
    strcpy(testTitle[1], "Blt Stretch  ");
    strcpy(testTitle[2], "Blt Fill     ");
    strcpy(testTitle[3], "Blt VSYNCH   ");     
    strcpy(testTitle[4], "Blt CPU RAM  ");
    strcpy(testTitle[5], "10000 Lines  ");

    GlobalMemoryStatusEx(&lpBuffer);

    ramMB      = (DWORD)(lpBuffer.ullTotalPhys / 1048576);
    percentMB  = lpBuffer.dwMemoryLoad;
    ramFree    = (DWORD)(lpBuffer.ullAvailPhys / 1048576);
    pageMB     = (DWORD)(lpBuffer.ullTotalPageFile / 1048576);
    pageFree   = (DWORD)(lpBuffer.ullAvailPageFile / 1048576);
    virtMB     = (lpBuffer.ullTotalVirtual / 1048576);
    virtFree   = (lpBuffer.ullAvailVirtual / 1048576);

    sprintf(meminfo1[0], "  Mbytes of physical memory    %c%d", 9, ramMB);
    sprintf(meminfo1[1], "  Percent of memory in use     %c%d", 9, percentMB);
    sprintf(meminfo1[2], "  Free physical memory Mbytes  %c%d", 9, ramFree);
    sprintf(meminfo1[3], "  Mbytes of paging file        %c%d", 9, pageMB);
    sprintf(meminfo1[4], "  Free Mbytes of paging file   %c%d", 9, pageFree);
    sprintf(meminfo1[5], "  User Mbytes of virtual space %c%ld", 9, virtMB);
    sprintf(meminfo1[6], "  Free user virtual Mbytes     %c%ld", 9, virtFree);

    
    percentUse  = percentMB;
    minFreeRam  = ramFree;
    minFreePage = pageFree;
    minFreeVirt = virtFree;

        
    // display window
    
    ShowWindow( hwnd, SW_SHOWMAXIMIZED ); 
    UpdateWindow( hwnd );
    return TRUE;

                        
}  // InitApp



 // WinMain - initialization, message loop

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow )
{
    MSG         msg;
    
    hPrevInstance = hPrevInstance;

    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    GetMessage( &msg, NULL, 0, 0 );

    if (autoRun)
    {
        if (testHwnd)   DestroyWindow(testHwnd);
        hcur = SetCursor(LoadCursor(NULL, IDC_WAIT));
        if (!setTests())
        {
            SetCursor(hcur);
               return FALSE;
        }
        SetCursor(hcur);
    }
     
    while(!bQuit)
    {
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            
           if (msg.message == WM_QUIT)
           {
               finish();
               bQuit = TRUE;
               break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return (int)msg.wParam;
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else if (phase > 0)
        {
            hcur = SetCursor(LoadCursor(NULL, IDC_WAIT));
            runTests();
            SetCursor(hcur);        
            phase = phase + 1;
            if (phase == 13)
            {
                fileNumber = fileNumber + 1;
                if (debug)
                {
                    fprintf (outfile, " Phase %2d Size %d after run ", phase, fileSize[fileNumber]); 
                    GetMemoryStatus();
                    fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
                    fflush (outfile);
                }
                if (fileNumber == lastFile)
                {
                    hcur = SetCursor(LoadCursor(NULL, IDC_WAIT));
                    runTests();
                    phase = 0;
                    SetCursor(hcur);
                    if (autoRun)
                    {
                        finish();
                        bQuit = TRUE;
                    }        
                }
                else 
                {
                    phase = 1;
                }
            }
        }
        else if (displayToDo)
        {
                displayWhatever(mainHWND);
                displayToDo = FALSE;
        }
        else
        {
           // make sure we go to sleep if we have nothing else to do
           WaitMessage();
        }
    }   
    return (int)msg.wParam;

} // WinMain


BOOL FAR PASCAL AboutDlgProc( HWND hwnd, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    lparam = lparam;                    // turn off warning

    switch( msg )
    {
    case WM_INITDIALOG:
        return( TRUE );

    case WM_COMMAND:
        if( LOWORD( wparam ) == IDOK )
        {
            EndDialog( hwnd, TRUE );
            return( TRUE );
        }
        break;
    }
    return( FALSE );

} // AboutDlgProc



BOOL setTests()
{
    HDC hdc;
    DWORD e;
    
    finish();

    escPress = FALSE;
    runDone = FALSE;
    doBlt = TRUE;
    iright = 0;
    idown  = 0;
    doWide = imageDim[0];
    doHigh = imageDim[0];
    imageWide = doWide;
    imageHigh = doHigh;

    outfile = fopen(logFile,"a+");  
    if (outfile == NULL)
    {
        if (MessageBox(mainHWND, "Cannot open results file - Exit ?", "Error",
             MB_ICONQUESTION | MB_YESNO) == IDYES) exit(0);
    }
    local_time();
    fprintf (outfile, " ####################################################################\n");
    fprintf (outfile, " %s %s\n", version, timeday);
    fprintf (outfile, "           Copyright Roy Longbottom 1999 - 2006\n\n");
    fprintf(outfile, "   Input Enlarge    Save    Load  Scroll  Scroll  Rotate     Use\n");
    fprintf(outfile, "   Image Display         Display /Repeat Overall  90 deg    Fast\n");
    fprintf(outfile, "  Mbytes    Secs    Secs    Secs   msecs  MB/Sec    Secs  BitBlt\n\n");
    fflush(outfile);   

    keyPress = GetAsyncKeyState(VK_ESCAPE);


    fileNumber = 0;
    hdc = GetDC(mainHWND);
    dataLength =   imageDim[fileNumber] * imageDim[fileNumber] * 3 + 54;
    sizeToRead =   fileSize[fileNumber];

    input1 = (char *)VirtualAlloc(NULL, sizeToRead + fileXtra, MEM_COMMIT, PAGE_READWRITE);
    if (input1 == NULL)
    {
        e = GetLastError();
        GetMemoryStatus();
        ReleaseDC(mainHWND, hdc);
        sprintf(failMessage, "VirtualAlloc 1 - Cannot allocate %d MB memory Error %d 0x%8.8lx",
                              sizeToRead / 1048576, e, e);
        return initFail();
    }
    inputX = input1;

    hLoadBitmap1 = (HBITMAP)LoadImage(GetModuleHandle(NULL), "B204x204", IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);   
    if (hLoadBitmap1 == NULL)
    {
        e = GetLastError();
        GetMemoryStatus();
        VirtualFree(input1, 0, MEM_RELEASE);
        input1 = NULL;
        ReleaseDC(mainHWND, hdc);
        sprintf(failMessage, "LoadImage 1 failed Error %d 0x%8.8lx", e, e);
        return initFail();
    }

    lpInputDIB1  = (BITMAPINFO*)(input1 + sizeof(BITMAPFILEHEADER));

    lpInputDIB1->bmiHeader.biSize = sizeof( BITMAPINFOHEADER );
    lpInputDIB1->bmiHeader.biWidth = 204;
    lpInputDIB1->bmiHeader.biHeight = 204;
    lpInputDIB1->bmiHeader.biPlanes = 1;
    lpInputDIB1->bmiHeader.biBitCount = 24;
    lpInputDIB1->bmiHeader.biCompression = BI_RGB;
    lpInputDIB1->bmiHeader.biSizeImage = dataLength - 54;
    lpInputDIB1->bmiHeader.biXPelsPerMeter = 0;
    lpInputDIB1->bmiHeader.biYPelsPerMeter = 0;
    lpInputDIB1->bmiHeader.biClrUsed = 0;
    lpInputDIB1->bmiHeader.biClrImportant = 0;

    GetDIBits(hdc, hLoadBitmap1, 0, 204, input1+54, lpInputDIB1, DIB_RGB_COLORS);

    if(hLoadBitmap1) DeleteObject( hLoadBitmap1);
    hLoadBitmap1 = NULL;

    hLoadBitmap1 = CreateDIBitmap(hdc, (LPBITMAPINFOHEADER)lpInputDIB1,
         CBM_INIT, ((LPBITMAPINFO)lpInputDIB1)->bmiColors,
         lpInputDIB1, DIB_RGB_COLORS );
    if (hLoadBitmap1 == NULL)
    {
        e = GetLastError();
        GetMemoryStatus();
        VirtualFree(input1, 0, MEM_RELEASE);
        input1 = NULL;
        ReleaseDC(mainHWND, hdc);
        sprintf(failMessage, "CreateDIBitmap 1 failed Error %d 0x%8.8lx", e, e);
        return initFail();
    }

    hdcLoad1 = CreateCompatibleDC(hdc);
    hdcLoadX = hdcLoad1;
    canPaint = TRUE;
    SelectObject( hdcLoad1, hLoadBitmap1 );

    fileNumber = 1;    
    dataLength =   imageDim[fileNumber] * imageDim[fileNumber] * 3 + 54;
    sizeToRead =   fileSize[fileNumber];

    input2 = (char *)VirtualAlloc(NULL, sizeToRead + fileXtra, MEM_COMMIT, PAGE_READWRITE);
    if (input2 == NULL)
    {
        e = GetLastError();
        GetMemoryStatus();
        ReleaseDC(mainHWND, hdc);
        sprintf(failMessage, "VirtualAlloc 2 - Cannot allocate %d MB memory Error %d 0x%8.8lx",
                              sizeToRead / 1048576, e, e);
        return initFail();
    }
    
    hLoadBitmap2 = (HBITMAP)LoadImage(GetModuleHandle(NULL), "B288x288", IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);   
    if (hLoadBitmap2 == NULL)
    {
        e = GetLastError();
        GetMemoryStatus();
        ReleaseDC(mainHWND, hdc);
        sprintf(failMessage, "LoadImage 2 failed Error %d 0x%8.8lx", e, e);
        return initFail();
    }

    lpInputDIB2  = (BITMAPINFO*)(input2 + sizeof(BITMAPFILEHEADER));

    lpInputDIB2->bmiHeader.biSize = sizeof( BITMAPINFOHEADER );
    lpInputDIB2->bmiHeader.biWidth = 288;
    lpInputDIB2->bmiHeader.biHeight = 288;
    lpInputDIB2->bmiHeader.biPlanes = 1;
    lpInputDIB2->bmiHeader.biBitCount = 24;
    lpInputDIB2->bmiHeader.biCompression = BI_RGB;
    lpInputDIB2->bmiHeader.biSizeImage = dataLength - 54;
    lpInputDIB2->bmiHeader.biXPelsPerMeter = 0;
    lpInputDIB2->bmiHeader.biYPelsPerMeter = 0;
    lpInputDIB2->bmiHeader.biClrUsed = 0;
    lpInputDIB2->bmiHeader.biClrImportant = 0;

    GetDIBits(hdc, hLoadBitmap2, 0, 288, input2+54, lpInputDIB2, DIB_RGB_COLORS);

    if(hLoadBitmap2) DeleteObject( hLoadBitmap2);
    hLoadBitmap2 = NULL;

    hLoadBitmap2 = CreateDIBitmap(hdc, (LPBITMAPINFOHEADER)lpInputDIB2,
         CBM_INIT, ((LPBITMAPINFO)lpInputDIB2)->bmiColors,
         lpInputDIB2, DIB_RGB_COLORS );
    if (hLoadBitmap2 == NULL)
    {
        e = GetLastError();
        GetMemoryStatus();
        ReleaseDC(mainHWND, hdc);
        sprintf(failMessage, "CreateDIBitmap 2 failed Error %d 0x%8.8lx", e, e);
        return initFail();
    }
    hdcLoad2 = CreateCompatibleDC( hdc );
    SelectObject( hdcLoad2, hLoadBitmap2 );
    firstOf2 = TRUE;
    ReleaseDC(mainHWND, hdc);
    runDone = TRUE;
    phase = 1;
    gotPixel = 1;
    fileNumber = 2;
    return TRUE;

}  // setTests


BOOL runTests()
{
    HDC hdc;
    DWORD e;

    displayOut1 = FALSE;
    if (phase == 1)  // *********** Allocate Output
    {
        useBlt = 0;
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        if (firstOf2) // ********** Which Input
        {
            inputX = input1;
            hdcLoadX = hdcLoad1;
            canPaint = TRUE;
            hLoadBitmapX = hLoadBitmap1;
            lpInputDIBX = lpInputDIB1;
        }
        else
        {
            inputX = input2;
            hdcLoadX = hdcLoad2;
            canPaint = TRUE;
            hLoadBitmapX = hLoadBitmap2;
            lpInputDIBX = lpInputDIB2;
        }
        start_time();
        sprintf(windowText, " BMPSpeed Enlarge %d x %d to %d x %d, Size %7.2f MB",
                               imageDim[fileNumber-2], imageDim[fileNumber-2],
                               imageDim[fileNumber-2]*2, imageDim[fileNumber-2]*2,
                               (double)(fileSize[fileNumber]) / 1048576);
        SetWindowText(mainHWND, windowText);
        if (!allocate1())   //  output1
        {
            return initFail();
        }
    }
    else if (phase == 2)  // *********** Enlarge inputX to Output
    {
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        if (!enlarge())
        {
            return initFail();
        }
        if (firstOf2)  //  ************** Free Input
        {
            canPaint = FALSE;
            if(hdcLoad1) DeleteDC( hdcLoad1 );
            hdcLoad1 = NULL;
            if(hLoadBitmap1) DeleteObject( hLoadBitmap1 );
            hLoadBitmap1 = NULL;
            if (input1) VirtualFree(input1, 0, MEM_RELEASE);
            input1 = NULL;
        }
        else
        {
            canPaint = FALSE;
            if(hdcLoad2) DeleteDC( hdcLoad2 );
            hdcLoad2 = NULL;
            if(hLoadBitmap2) DeleteObject( hLoadBitmap2 );
            hLoadBitmap2 = NULL;
            if (input2) VirtualFree(input2, 0, MEM_RELEASE);
            input2 = NULL;
        }
    }
    else if (phase == 3)  // *********** CreateDIB Output
    {
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        sprintf (progressText, "Create BitMap %5.0f MB", (double)(sizeToSave) / 1048576);
        progressHwnd = CreateDialog( MyInstance, "Progress2", mainHWND, (DLGPROC)ProgressProc2);  
        hdc = GetDC(mainHWND);
    
        lpOutputDIB  = (BITMAPINFO*)(output1 + sizeof(BITMAPFILEHEADER));
        hOutBitmap = CreateDIBitmap(hdc, (LPBITMAPINFOHEADER)lpOutputDIB,
             CBM_INIT, ((LPBITMAPINFO)lpOutputDIB)->bmiColors,
             lpOutputDIB, DIB_RGB_COLORS ); 
        if (hOutBitmap == NULL)
        {
            doBlt2 = FALSE;
            pBuf2 = output1 +  54;
            lpbi2 = (LPBITMAPINFOHEADER)lpOutputDIB;
        }
        else
        {
            doBlt2 = TRUE;
            useBlt = useBlt + 1;
            hdcOut = CreateCompatibleDC( hdc );
            if (hdcOut == NULL)
            {
                e = GetLastError();
                GetMemoryStatus();
                ReleaseDC(mainHWND, hdc);
                if (progressHwnd)   DestroyWindow(progressHwnd);
                sprintf(failMessage, "CreateCompatibleDC 3 Failed Error %d 0x%8.8lx", e, e);
                return initFail();
            }
            SelectObject( hdcOut, hOutBitmap );
        }
        if (progressHwnd)   DestroyWindow(progressHwnd);
        gotPixel = gotPixel | GetPixel(hdc, 2, 1);
        ReleaseDC(mainHWND, hdc);
    }   
    else if (phase == 4)  // *********** Display Output
    {
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        hdc = GetDC(mainHWND);
        sprintf (progressText, "Display Image %5.0f MB", (double)(sizeToSave) / 1048576);
        progressHwnd = CreateDialog( MyInstance, "Progress2", mainHWND, (DLGPROC)ProgressProc2);  
        displayOut1 = TRUE;
        if (!displayBMP3(hdc))
        {
            if (progressHwnd)   DestroyWindow(progressHwnd);
            ReleaseDC(mainHWND, hdc);
            return initFail();
        }
        if (progressHwnd)   DestroyWindow(progressHwnd);
        gotPixel = gotPixel | GetPixel(hdc, 2, 1);
        end_time();
        ReleaseDC(mainHWND, hdc);
        results[fileNumber][0] = secs;
    }
    else if (phase == 5)  // *********** Save & Free Output
    {
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        start_time();
        sprintf(windowText, " BMPSpeed Saving %s, Size %6.1f MB", fileToSave,
                                     (double)(sizeToSave) / 1048576);
        SetWindowText(mainHWND, windowText);
        if (!saveBMP())
        {
            return initFail();
        }
        displayOut1 = FALSE;
        if(hdcOut) DeleteDC( hdcOut );
        hdcOut = NULL;
        if(hOutBitmap) DeleteObject(hOutBitmap);
        hOutBitmap = NULL;             
        VirtualFree(output1, 0, MEM_RELEASE);
        output1 = NULL;
        hdc = GetDC(mainHWND);
        gotPixel = gotPixel | GetPixel(hdc, 2, 1);
        ReleaseDC(mainHWND, hdc);
        end_time();
        results[fileNumber][1] = secs;
    }
    else if (phase == 6)  // *********** Load Input
    {    
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        start_time();
        sprintf(windowText, " BMPSpeed Loading %s, Size %6.1f MB", fileToSave,
                                     (double)(fileSize[fileNumber]) / 1048576);
        SetWindowText(mainHWND, windowText);
        sizeToRead =   fileSize[fileNumber];

        hdc = GetDC(mainHWND);
        if (firstOf2)
        {
            if (fileNumber == lastFile - 1)
            {
                canPaint = FALSE;
                if(hdcLoad2) DeleteDC( hdcLoad2 );
                hdcLoad2 = NULL;
                if(hLoadBitmap2) DeleteObject( hLoadBitmap2 );
                hLoadBitmap2 = NULL;
                if (input2) VirtualFree(input2, 0, MEM_RELEASE);
                input2 = NULL;
            }
            input1 = (char *)VirtualAlloc(NULL, sizeToRead, MEM_COMMIT, PAGE_READWRITE);
            if (input1 == NULL)
            {
                e = GetLastError();
                GetMemoryStatus();
                ReleaseDC(mainHWND, hdc);
                sprintf(failMessage, "VirtualAlloc 4 - Cannot allocate %d MB memory Error %d 0x%8.8lx",
                                     sizeToRead / 1048576, e, e);
                return initFail();
            }
            inputX = input1;
        
            strcpy(fileToLoad, fileToSave);
            dataAddress = input1;
            
            if (!loadInputBMP())
            {
                ReleaseDC(mainHWND, hdc);
                return initFail();
            }        
            lpInputDIB1  = (BITMAPINFO*)(input1 + sizeof(BITMAPFILEHEADER));
            lpInputDIBX = lpInputDIB1;
            
            sprintf (progressText, "Create BitMap %5.0f MB", (double)(sizeToRead) / 1048576);
            progressHwnd = CreateDialog( MyInstance, "Progress2", mainHWND, (DLGPROC)ProgressProc2);  
            hLoadBitmap1 = CreateDIBitmap(hdc, (LPBITMAPINFOHEADER)lpInputDIB1,
                 CBM_INIT, ((LPBITMAPINFO)lpInputDIB1)->bmiColors,
                 lpInputDIB1, DIB_RGB_COLORS ); 
            if (hLoadBitmap1 == NULL)
            {
                doBlt = FALSE;
                pBuf = input1 +  54;
                lpbi = (LPBITMAPINFOHEADER)lpInputDIB1;
            }
            else
            {
                doBlt = TRUE;
                useBlt = useBlt + 1;
                hdcLoad1 = CreateCompatibleDC( hdc );
                if (hdcLoad1 == NULL)
                {
                    e = GetLastError();
                    if (progressHwnd)   DestroyWindow(progressHwnd);
                    GetMemoryStatus();
                    ReleaseDC(mainHWND, hdc);
                    sprintf(failMessage, "CreateCompatibleDC 1 Failed Error %d 0x%8.8lx", e, e);
                    return initFail();
                }
                SelectObject( hdcLoad1, hLoadBitmap1 );
                hdcLoadX = hdcLoad1;
                canPaint = TRUE;
                hLoadBitmapX = hLoadBitmap1;
            }

        }
        else
        {
            if (fileNumber == lastFile - 1)
            {
                canPaint = FALSE;
                if(hdcLoad1) DeleteDC( hdcLoad1 );
                hdcLoad1 = NULL;
                if(hLoadBitmap1) DeleteObject( hLoadBitmap1 );
                hLoadBitmap1 = NULL;
                if (input1) VirtualFree(input1, 0, MEM_RELEASE);
                input1 = NULL;
            }
            input2 = (char *)VirtualAlloc(NULL, sizeToRead, MEM_COMMIT, PAGE_READWRITE);
            if (input2 == NULL)
            {
                e = GetLastError();
                GetMemoryStatus();
                ReleaseDC(mainHWND, hdc);
                sprintf(failMessage, "VirtualAlloc 5 - Cannot allocate %d MB memory Error %d 0x%8.8lx",
                                     sizeToRead / 1048576, e, e);
                return initFail();
            }
            inputX = input2;
        
            strcpy(fileToLoad, fileToSave);
            dataAddress = input2;
            
            if (!loadInputBMP())
            {
                ReleaseDC(mainHWND, hdc);
                return initFail();
            }
            lpInputDIB2  = (BITMAPINFO*)(input2 + sizeof(BITMAPFILEHEADER));
            lpInputDIBX = lpInputDIB2;
         
            sprintf (progressText, "Create BitMap %5.0f MB", (double)(sizeToRead) / 1048576);
            progressHwnd = CreateDialog( MyInstance, "Progress2", mainHWND, (DLGPROC)ProgressProc2);  
            hLoadBitmap2 = CreateDIBitmap(hdc, (LPBITMAPINFOHEADER)lpInputDIB2,
                 CBM_INIT, ((LPBITMAPINFO)lpInputDIB2)->bmiColors,
                 lpInputDIB2, DIB_RGB_COLORS );
            if (hLoadBitmap2 == NULL)
            {
                doBlt = FALSE;
                pBuf = input2 +  54; 
                lpbi = (LPBITMAPINFOHEADER)lpInputDIB2;
            }
            else
            {
                doBlt = TRUE;
                useBlt = useBlt + 1;
                hdcLoad2 = CreateCompatibleDC( hdc );
                if (hdcLoad2 == NULL)
                {
                    e = GetLastError();
                    GetMemoryStatus();
                    if (progressHwnd)   DestroyWindow(progressHwnd);
                    ReleaseDC(mainHWND, hdc);
                    sprintf(failMessage, "CreateCompatibleDC 2 Failed Error %d 0x%8.8lx", e, e);
                    return initFail();
                }
                SelectObject( hdcLoad2, hLoadBitmap2 );
                hdcLoadX = hdcLoad2;
                canPaint = TRUE;
                hLoadBitmapX = hLoadBitmap2;
           }
        }
        if (progressHwnd)   DestroyWindow(progressHwnd);    
        gotPixel = gotPixel | GetPixel(hdc, 2, 1);
        ReleaseDC(mainHWND, hdc);
        if (mintest)
        {
            end_time();
            results[fileNumber][2] = secs;
        }
    }
    else if (phase == 7 && mintest == FALSE)  // *********** Display Input
    {
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        hdc = GetDC(mainHWND);
        sprintf (progressText, "Display Image %5.0f MB", (double)(sizeToRead) / 1048576);
        progressHwnd = CreateDialog( MyInstance, "Progress2", mainHWND, (DLGPROC)ProgressProc2);
        if (!displayBMP1(hdc))
        {
            if (progressHwnd)   DestroyWindow(progressHwnd);
            ReleaseDC(mainHWND, hdc);
            return initFail();
        }
        if (progressHwnd)   DestroyWindow(progressHwnd);
        gotPixel = gotPixel | GetPixel(hdc, 2, 1);
        end_time();
        results[fileNumber][2] = secs;   
        ReleaseDC(mainHWND, hdc);
    }
    else if (phase == 8 && mintest == FALSE)  // *********** Scroll Input
    {    
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        sprintf(windowText, " BMPSpeed Scrolling %6.1f MB Image",
                                 (double)(sizeToRead) / 1048576);
        SetWindowText(mainHWND, windowText);

        if (!scroll())  // gets secs
        {
            return initFail();
        }
        results[fileNumber][3] = secs / (double)passCount;   
        results[fileNumber][4] = (double)bitsaPixel / 8
                               * (double)doWide / 1000
                               * (double)doHigh / 1000
                               / results[fileNumber][3];

    }
    else if (phase == 9 && mintest == FALSE)  // *********** Allocate Output
    {    
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        start_time();
        sprintf(windowText, " BMPSpeed Rotate %d x %d, Size %7.2f MB",
                               imageDim[fileNumber], imageDim[fileNumber],         
                               (double)(sizeToRead) / 1048576);
        SetWindowText(mainHWND, windowText);
        if (!allocate1())
        {
            return initFail();
        }
    }
    else if (phase == 10 && mintest == FALSE)  // *********** Copy/Rotate
    {    
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        if (!rotate())
        {
            return initFail();
        }
    }
    else if (phase == 11 && mintest == FALSE)  // *********** Display Rotated Output Free
    { 
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        displayOut1 = TRUE;
        if (!displayBMP2())
        {
            return initFail();
        }
        displayOut1 = FALSE;
        VirtualFree(output1, 0, MEM_RELEASE);
        output1 = NULL;
        end_time();
        results[fileNumber][5] = secs;       
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d after displayBMP2 ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        sprintf (progressText, "Display Rotated Image 1 Second");
        progressHwnd = CreateDialog( MyInstance, "Progress2", mainHWND, (DLGPROC)ProgressProc2);  
        hdc = GetDC(mainHWND);
        gotPixel = gotPixel | GetPixel(hdc, 2, 1);
        ReleaseDC(mainHWND, hdc);
        Sleep(1000);
        if (progressHwnd)   DestroyWindow(progressHwnd);
    }
    else if (phase == 12)
    {        
        if (firstOf2)
        {
            firstOf2 = FALSE;
        }
        else
        {
            firstOf2 = TRUE;
        }  
        hdc = GetDC(mainHWND);

        if (!displayBMP1(hdc))
        {
            ReleaseDC(mainHWND, hdc);
            return initFail();
        }
        gotPixel = gotPixel | GetPixel(hdc, 2, 1);
        ReleaseDC(mainHWND, hdc);

        fprintf(outfile, " %7.1f %7.2f %7.2f %7.2f %7.1f %7.1f %7.2f %6d\n",
                                  (double)(sizeToRead) / 1048576,
                                  results[fileNumber][0],
                                  results[fileNumber][1],
                                  results[fileNumber][2],
                                  results[fileNumber][3] * 1000,
                                  results[fileNumber][4],
                                  results[fileNumber][5],
                                  useBlt);

        if (gotPixel == 0) fprintf(outfile, "\n");
        fflush(outfile);   
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d after fprint ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
    }
    else if (phase == 13)
    {
        if (debug)
        {
            fprintf (outfile, " Phase %2d Size %d ", phase, fileSize[fileNumber]); 
            GetMemoryStatus();
            fprintf (outfile, " MB Free RAM %d PageFile %d\n", ramFree, pageFree);
            fflush (outfile);
        }
        endPrint();
        displayWhatever(mainHWND);
    }
    return TRUE;
        
} // runTests

VOID endPrint()
{
    sprintf(windowText, "  %s", TITLE);
    SetWindowText(mainHWND, windowText);

    fprintf(outfile, "\n%s\n", configdata[1]);
    fprintf(outfile, "%s\n", configdata[2]);
    fprintf(outfile, "%s\n", configdata[5]);
    fprintf(outfile, "%s\n", configdata[6]);
    fprintf(outfile, "  Memory Status Maximum Use\n");
    fprintf(outfile, "  Mbytes of physical memory    %d\n", ramMB);
    fprintf(outfile, "  Percent of memory in use     %d\n", percentUse);
    fprintf(outfile, "  Free physical memory Mbytes  %d\n", minFreeRam);
    fprintf(outfile, "  Mbytes of paging file        %d\n", pageMB);
    fprintf(outfile, "  Free Mbytes of paging file   %d\n", minFreePage);
    fprintf(outfile, "  User Mbytes of virtual space %ld\n", virtMB);
    fprintf(outfile, "  Free user virtual Mbytes     %ld\n", minFreeVirt);
     
    sprintf(meminfo2[0], "  Mbytes of physical memory    %c%d", 9, ramMB);
    sprintf(meminfo2[1], "  Percent of memory in use     %c%d", 9, percentUse);
    sprintf(meminfo2[2], "  Free physical memory Mbytes  %c%d", 9, minFreeRam);
    sprintf(meminfo2[3], "  Mbytes of paging file        %c%d", 9, pageMB);
    sprintf(meminfo2[4], "  Free Mbytes of paging file   %c%d", 9, minFreePage);
    sprintf(meminfo2[5], "  User Mbytes of virtual space %c%ld", 9, virtMB);
    sprintf(meminfo2[6], "  Free user virtual Mbytes     %c%ld", 9, minFreeVirt);

    fprintf(outfile, "  Screen setting %d x %d x %d bits = %4.1f MB\n", screenWide, screenHigh, bitsaPixel,
                       (double)bitsaPixel / 8 * (double)screenWide / 1000 * (double)screenHigh / 1000 );
    if (strlen(commandLine)) fprintf(outfile, "  Batch Commands: %s\n", commandLine); 

    if (writeThru)
    {
        fprintf (outfile, "\n  Original CreateFile using FILE_FLAG_WRITE_THROUGH \n");
    }
    fflush (outfile);
    local_time();
    fprintf (outfile, "\n                    End at %s \n", timeday);
    fclose(outfile);
    testsDone = TRUE;
    
} // endPrint


void displayWhatever(HWND hwnd)
{
    if (testHwnd)   DestroyWindow(testHwnd);
    if (resultHwnd) DestroyWindow(resultHwnd);
    if (selectHwnd) DestroyWindow(selectHwnd);
    if (useTestPanel)
    {
        testHwnd = CreateDialog( MyInstance,"DIALOG_8", hwnd, (DLGPROC)TestPanel);
    }
    else
    {
        if (showIndex)
        {
            selectHwnd = CreateDialog( MyInstance,"DIALOG_10", hwnd, (DLGPROC)ResultSelect);
        }
        else
        {
            resultHwnd = CreateDialog( MyInstance,"DIALOG_9", hwnd, (DLGPROC)ResultData);
        }
    }
}

BOOL FAR PASCAL ProgressProc( HWND hdlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    lparam = lparam;
    wparam = wparam;
    HWND     hwndName;
                               
    
    switch( msg )
    {
     case WM_INITDIALOG:
         hwndName = GetDlgItem(hdlg, TEST_BOX);
         SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) progressText); 
         return( TRUE );

     case WM_COMMAND:
          break;
    }
    return FALSE;
}

BOOL FAR PASCAL ProgressProc2( HWND hdlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    lparam = lparam;
    wparam = wparam;
    HWND     hwndName;
                               
    
    switch( msg )
    {
     case WM_INITDIALOG:
         hwndName = GetDlgItem(hdlg, NOTE_BOX);
         SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) progressText); 
         return( TRUE );

     case WM_COMMAND:
          break;
    }
    return FALSE;
}

BOOL FAR PASCAL TestPanel( HWND hdlg, UINT msg,
                                WPARAM wparam, LPARAM lparam )
{
    wparam = wparam;
    lparam = lparam;
    hdlg = hdlg;
    int FAR  tabs[11] = {120, 130, 140};
    HWND     hwndName;
    int i;
    char wdata[30];

     if (writeThru)
     {
         strcpy(note4, " Use WRITE_THROUGH flag. ");
     }
     else
     {
         strcpy(note4, " No WRITE_THROUGH flag. ");
     }
     strcat(note4, note1);
     
     switch( msg )
     {
       case WM_INITDIALOG:
       
            SendDlgItemMessage(hdlg, INFO, LB_SETTABSTOPS, (WPARAM) 3, (LPARAM) (int FAR*) tabs);
            hwndName = GetDlgItem(hdlg, TEST_NOTES);
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) configdata[1]);
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) configdata[2]);            
            if (strlen(note2) > 1) SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) note2); 
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) note4); 
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) note3); 
            hwndName = GetDlgItem(hdlg, INIT_SET);    
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) startSettings);
            hwndName = GetDlgItem(hdlg, INFO);
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) "  Memory Status Start");
            for (i=0; i<7; i++)
            {
                SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) meminfo1[i]);
            }
            SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) diskinfo);
            
            if (runDone)
            {
                SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) " ");
                SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) "  Memory Status Maximum Use");
                for (i=1; i<7; i++)
                {
                    SendMessage(hwndName, LB_ADDSTRING, 0, (LPARAM) meminfo2[i]);
                }
            }
            for (i=3; i<13; i++)
            {
                sprintf(wdata, "%8.1f", (double)fileSize[i] / 1048576);
                SendDlgItemMessage(hdlg, MAX_SIZE, CB_ADDSTRING, 0, (LPARAM) wdata);
            }
            SendDlgItemMessage(hdlg, MAX_SIZE, CB_SETCURSEL, lastFile-4, 0L);
/*
            if (maxSize > (int)freeDiskMB / 2)
            {
                sprintf (failMessage, "WARNING: Insufficient Disk Space\n"
                                      "Reduce Maxumum Image Size");
                MessageBox( mainHWND, failMessage, "WARNING", MB_OK | MB_ICONWARNING );
            }
*/
            if (writeThru)
            {
                CheckDlgButton(hdlg, WRITE_THROUGH, BST_CHECKED);
            }
            return( TRUE );

        case WM_COMMAND:

            switch (LOWORD (wparam))
            {
                case TEST_IT:
                  PostMessage(mainHWND, WM_COMMAND, MENU_RUN, 0);
                 break;

                case EXIT_BUTTON:
                 PostMessage(mainHWND, WM_COMMAND, MENU_EXIT, 0);
                 break;

               case SEE_RESULTS:
                 PostMessage(mainHWND, WM_COMMAND, MENU_DISPLAY, 0);
                 break;

               case FIND_RESULTS:
                 PostMessage(mainHWND, WM_COMMAND, MENU_SELECT, 0);
                 break;
            }
            if( HIWORD( wparam ) == BN_CLICKED)
            {
                if (IsDlgButtonChecked(hdlg, WRITE_THROUGH) == BST_CHECKED)
                {
                    writeThru = TRUE;
                }
                else
                {
                    writeThru = FALSE;
                }
                displayWhatever(mainHWND);
            }
            if( HIWORD( wparam ) == CBN_SELCHANGE)
            {
                switch (LOWORD (wparam))
                {
                    case MAX_SIZE:
                      lastFile = (int)SendDlgItemMessage(hdlg, MAX_SIZE, CB_GETCURSEL, 0, 0)+4;
                      maxSize = fileSize[lastFile - 1] / 1048576;
/*
                    if (maxSize > (int)freeDiskMB / 2)
                    {
                        sprintf (failMessage, "WARNING: Insufficient Disk Space\n"
                                              "Reduce Maxumum Image Size");
                        MessageBox( mainHWND, failMessage, "WARNING", MB_OK | MB_ICONWARNING );
                    }
*/
                }
            } 
     }
     return( FALSE );
    
} // TestPanel



BOOL findLastLog()
{
   FILE *infile;
   char buf[350];
   int  i;

   resCount = 0;
   infile = fopen(thisLog,"r");
   if (infile == NULL)
   {
       MessageBox(NULL, "Cannot open log file", thisLog, MB_ICONWARNING | MB_OK);
       return FALSE;
   }
   while (fgets(buf, 120, infile) != NULL)
   {
       if (strnicmp(buf, " BMP Image Save/Load/Edit Test", 30) == 0)
       {
           resCount = resCount + 1;
       }   
   }
   fclose(infile);
   if (useLast)
   {
       readData(resCount);
   }
   else
   {
        for (i = 0; i<resCount+1; i++)
        {
             readData(i);
        }
   }
   return TRUE;
   
} // findLastLog

static BOOL GetFileName()
{
    static char    filterList1[] = "File (*.txt)" \
                                        "\0" \
                                        "*.txt" \
                                        "\0\0";
    OPENFILENAME        of;
    int                 rc;
    
    memset( &of, 0, sizeof( OPENFILENAME ) );
    of.lStructSize = sizeof( OPENFILENAME );
    of.hwndOwner = mainHWND;
    of.lpstrFilter = (LPSTR) filterList1;
    of.lpstrDefExt = "";
    of.nFilterIndex = 1L;
    of.lpstrFile = thisLog;
    of.nMaxFile = _MAX_PATH;
    of.lpstrTitle = NULL;
    of.Flags = OFN_HIDEREADONLY;

    rc = GetOpenFileName( &of );
    strcpy(thisLog, of.lpstrFile);
   
    return( rc );
    
} // GetFileName

BOOL FAR PASCAL ResultData( HWND hdlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
   int i, k;
   char buf[200];
   char buf2[260];
   int FAR  tabs[11] = {20, 40, 60, 80, 100, 120, 140, 160, 180, 200, 220};

   hdlg = hdlg;
   wparam = wparam;
   lparam = lparam;

   switch( msg )
   {
       case WM_INITDIALOG:
        SendDlgItemMessage(hdlg, RESULTS_BOX, LB_SETTABSTOPS, (WPARAM) 8, (LPARAM) (int FAR*) tabs);
        sprintf(buf2, "    Input %cEnlarge %c  Save %c   Load %c Scroll %c Scroll  %cRotate %c  Use", 9, 9, 9, 9, 9, 9, 9);
        SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) buf2);
        sprintf(buf2, "    Image %cDisplay %c  Only  %cDisplay %c/Repeat %cOverall  %c90 deg %c Fast", 9, 9, 9, 9, 9, 9, 9);
        SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) buf2);
        sprintf(buf2, "   Mbytes %cSeconds %cSeconds %cSeconds %c  msecs %c MB/Sec %cSeconds %c BitBlt", 9, 9, 9, 9, 9, 9, 9);
        SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) buf2);
        for (i = 0; i<resultsIn; i++)
        {
           sprintf(buf, " ");
           for (k=0; k<7; k++)
           {
               sprintf (buf2, "%8.2f%c", results[i][k], 9);
               strcat(buf, buf2); 
           }
           sprintf (buf2, "%8d%c", (int)results[i][7], 9);
           strcat(buf, buf2); 
           SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) buf);
        }
        sprintf (buf2, "    ");
        SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) buf2);
        for (i=0; i<2; i++)
        {
            SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) config[i]);
        }
        for (i=0; i<10; i++)
        {
            SendDlgItemMessage(hdlg, RESULTS_BOX, LB_ADDSTRING, 0, (LPARAM) meminfo3[i]);
        }

        return( TRUE );         
     
       case WM_COMMAND:
          if( HIWORD( wparam ) == BN_CLICKED)
          {
              switch (LOWORD (wparam))
              {                 
                  case IDOK:
                     DestroyWindow(resultHwnd);
                     useTestPanel = TRUE;
                     displayToDo = TRUE;
                     break;
              }
          }
          return TRUE;      
   }
   return FALSE;
   
} // ResultData

BOOL FAR PASCAL ResultSelect( HWND hdlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
   int i;
   int FAR  tabs[4] = {80, 260, 270, 280};
   int index = 0;


   hdlg = hdlg;
   wparam = wparam;
   lparam = lparam;

   switch( msg )
   {
       case WM_INITDIALOG:
        SendDlgItemMessage(hdlg, ENTRY_BOX, LB_SETTABSTOPS, (WPARAM) 4, (LPARAM) (int FAR*) tabs);
        SendDlgItemMessage(hdlg, ENTRY_BOX, LB_SETHORIZONTALEXTENT, 1200, (LPARAM)NULL);
        for (i = 1; i<resCount+1; i++)
        {
           SendDlgItemMessage(hdlg, ENTRY_BOX, LB_ADDSTRING, 0, (LPARAM) details[i]);
        }
        SendDlgItemMessage(hdlg, ENTRY_BOX, LB_SETCURSEL, 0, 0L);
        return( TRUE );         
     
       case WM_COMMAND:
          if( HIWORD( wparam ) == BN_CLICKED)
          {
              switch (LOWORD (wparam))
              {                 
                  case IDOK:
                     index = (int)SendDlgItemMessage(hdlg, ENTRY_BOX, LB_GETCURSEL, 0, 0);
                     DestroyWindow(selectHwnd);
                     useTestPanel = FALSE;
                     showIndex = FALSE;
                     if (!readData(index+1)) useTestPanel = TRUE;
                     displayToDo = TRUE;
                     break;

                  case IDCANCEL:
                     DestroyWindow(selectHwnd);
                     useTestPanel = TRUE;
                     displayToDo = TRUE;
                     break;
              }
          }
          return TRUE;      
   }
   return FALSE;
   
} // ResultSelect

BOOL readData(int resCount2)
{
   FILE *infile;
   char buf[350];
   char buf2[40];
   char buf3[40];
   char b[11][10];
   int  i, j, c;
   int  newCount = 0;
   BOOL fin = FALSE;
   
   for (i=0; i<40; i++)
   {
       wideUsed[i] = 0;
       highUsed[i] = 0;
       bitsUsed[i] = 0;
       for (j=0; j<10; j++)
       {
           results[i][j] = 0;
       }
   }
   sprintf(config[0], "  OS?");
   sprintf(config[1], "  CPU?");
   sprintf(config[2], "  Memory?");
   sprintf(config[3], "  end time?");

   infile = fopen(thisLog,"r");
   if (infile == NULL)
   {
       MessageBox(NULL, "Cannot open log file", thisLog, MB_ICONWARNING | MB_OK);
       return FALSE;
   }
   while (newCount < resCount2 && fgets(buf, 120, infile) != NULL)
   {
       if (strnicmp(buf, " BMP Image Save/Load/Edit Test", 30) == 0)
       {
           newCount = newCount + 1;
       }   
   }
   c = 0;
   resultsIn = 0;
   if (resCount2 > 0)
   {
       while (!fin && fgets(buf, 120, infile) != NULL)
       {
           c = c + 1;
           if (c > 7)
           {
               if (strlen(buf) < 56)
               {
                   fin = TRUE;
               }
               else
               {
                   sscanf(buf, "%8c%8c%8c%8c%8c%8c%8c%7c",
                                b[0],  b[1],  b[2],  b[3],  b[4],  b[5], b[6], b[7]);
                   for (i=0; i< 8; i++)
                   {
                       results[resultsIn][i] = strtod(b[i], NULL);
                   }
                   resultsIn = resultsIn + 1;
               } 
           }
       }
       c = 0;
       fin = FALSE;
       while (!fin && fgets(buf, 120, infile) != NULL)
       {
           strcpy (buf2, "                                     ");
           strcpy (buf3, "                                     "); 
           c = c + 1;
           if (strlen(buf) > 1)
           {
               if (strlen(buf) > 31)
               {
                   sscanf(buf, "%31c", buf2);

                   for (i=3; i<31; i++)
                   {
                       if (buf2[i] == 32) buf2[i]  = 95;
                   }
                   for (i=0; i<lstrlen(buf)-31; i++)
                   {
                       buf3[i] = buf[i+31];
                   }
                   buf3[i-1] = NULL;
                   
               }
               if (c == 1 && strnicmp(buf, "  Win", 5) == 0)
               {
                   strcpy (config[0], buf);
                   config[0][strlen(buf)-1] = NULL;
               }
               if (c == 2 && strnicmp(buf, "  CPU", 5) == 0)
               {
                   strcpy (config[1], buf);
                   config[1][strlen(buf)-1] = NULL;
               }
               if (c == 3 && strnicmp(buf, "  Mem", 5) == 0)
               {
                   strncpy (meminfo3[0], buf, strlen(buf)-1);
                   meminfo3[0][strlen(buf)-1] = NULL;
               }
               if (c == 4 && strnicmp(buf, "  Mby", 5) == 0)
               {
                   sprintf(meminfo3[1], "%s%c%s", buf2, 9, buf3);
               }
               if (c == 5 && strnicmp(buf, "  Per", 5) == 0)
               {
                   sprintf(meminfo3[2], "%s%c%s", buf2, 9, buf3);
               }
               if (c == 6 && strnicmp(buf, "  Fre", 5) == 0)
               {
                   sprintf(meminfo3[3], "%s%c%s", buf2, 9, buf3);
               }
               if (c == 7 && strnicmp(buf, "  Mby", 5) == 0)
               {
                   sprintf(meminfo3[4], "%s%c%s", buf2, 9, buf3);
               }
               if (c == 8 && strnicmp(buf, "  Fre", 5) == 0)
               {
                   sprintf(meminfo3[5], "%s%c%s", buf2, 9, buf3);
               }
               if (c == 9 && strnicmp(buf, "  Use", 5) == 0)
               {
                   sprintf(meminfo3[6], "%s%c%s", buf2, 9, buf3);
               }
               if (c == 10 && strnicmp(buf, "  Fre", 5) == 0)
               {
                   sprintf(meminfo3[7], "%s%c%s", buf2, 9, buf3);
               }
               if (c == 11 && strnicmp(buf, "  Scr", 5) == 0)
               {
                   strncpy (meminfo3[8], buf, strlen(buf)-1);
                   meminfo3[8][strlen(buf)-1] = NULL;
               }
               if (c == 13 && strnicmp(buf, "                    End", 23) == 0)
               {
                   for (i=0; i<lstrlen(buf)-18; i++)
                   {
                       buf3[i] = buf[i+18];
                   }
                   buf3[i] = NULL;
                   for (i=6; i<lstrlen(buf3-1); i++)
                   {
                       config[3][i-6] = buf3[i];
                   }
                   strncpy (meminfo3[9], buf3, strlen(buf3)-1);
                   meminfo3[9][strlen(buf3)-1] = NULL;
                   fin = TRUE;
               }
           }
       }
       if (!useLast && resCount2 < 51)
       {
           if (strlen(details[resCount2]) + strlen(config[2]) < 200)
           {
               sprintf(details[resCount2], " %s%c%s%c%s", config[3], 9, config[0], 9, config[1]);
           }
           else
           {
               sprintf(details[resCount2], " %s%c%s%c  CPU?", config[3], 9, config[0], 9);
           }
       }                  
   }
   fclose(infile);

   return TRUE;

} // readData 

BOOL addVesa()
{


    FILE *infile;
    FILE *outfile;
    char buf2[100];
    char buffer[100];

    if (escPress) return TRUE;
    
    // What VESA

    system("vesa.exe"); // ccccc
    
    outfile = fopen(logFile, "a+");
    if (outfile == NULL)
    {
          MessageBox(mainHWND, "Cannot open log file", logFile,
                             MB_ICONINFORMATION | MB_OK);
          return FALSE;
    }
    if (strlen(note2) > 1) fprintf(outfile, " %s\n\n", note2);
    infile = fopen("whatvesa.txt", "r");
    if (infile == NULL)
    {
          MessageBox(mainHWND, "Cannot open file whatvesa.txt", "Error",
                             MB_ICONINFORMATION | MB_OK);
          fclose(outfile);
          return FALSE;
    }
    while( fgets( buffer, 80, infile ) != NULL )
    {
        lstrcpyn(buf2, buffer, lstrlen(buffer));
        fprintf (outfile, " %s\n", buf2);
    }
    fclose(infile);
    fclose(outfile);

    return TRUE;
    
} // addVesa

BOOL loadInputBMP()
{
    HANDLE             hFile;
    DWORD              BytesRead, s;
    int               i;
    DWORD              e;

    if (writeThru)
    {
        hFile = CreateFile(fileToLoad, GENERIC_READ,
                 FILE_SHARE_READ | FILE_SHARE_WRITE,
                 NULL, OPEN_EXISTING,
                 FILE_ATTRIBUTE_NORMAL | FILE_FLAG_WRITE_THROUGH
                                       | FILE_FLAG_SEQUENTIAL_SCAN
                                       | FILE_FLAG_NO_BUFFERING, NULL);
    }
    else
    {
        hFile = CreateFile(fileToLoad, GENERIC_READ,
                 FILE_SHARE_READ | FILE_SHARE_WRITE,
                 NULL, OPEN_EXISTING,
                 FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN
                                       | FILE_FLAG_NO_BUFFERING, NULL);
    }
    if (hFile == INVALID_HANDLE_VALUE)
    {
        e = GetLastError();
        GetMemoryStatus();
        sprintf(failMessage, "Create Read Invalid Handle file %s Error %d 0x%8.8lx", fileToLoad, e, e);
        return FALSE;
    }
    s = sizeToRead / filePasses[fileNumber];

    GetMemoryStatus();

    sprintf (progressText, "Reading File %s, Size %5.0f MB, Free RAM %d MB",
                     fileToLoad, (double)(sizeToRead) / 1048576,
                     ramFree);
    pbcount = filePasses[fileNumber];

    if (!startProgress())
    {
        e = GetLastError();
        GetMemoryStatus();
        CloseHandle( hFile );
        sprintf(failMessage, "startProgress failed Error %d 0x%8.8lx", e, e);
        return FALSE;
    }

    pbcount = 1; 
    SendMessage(hwndPB, PBM_SETSTEP, (WPARAM) pbcount, 0);

    for (i=0; i<filePasses[fileNumber]; i++)
    {
        if (ReadFile(hFile, dataAddress+s*i, s, &BytesRead, NULL) == 0)
        {
            e = GetLastError();
            GetMemoryStatus();
            sprintf(failMessage, "ReadFile %s failed, sizeToRead %d Error %d 0x%8.8lx",
                                 fileToLoad, sizeToRead, e, e);
            CloseHandle( hFile ); 
            endProgress();
            return FALSE;
        }
    
        if (imageDim[fileNumber] < 290) s = dataLength;
        if (s != BytesRead)
        {
            e = GetLastError();
            GetMemoryStatus();
            sprintf(failMessage, " Failed to load file %s\n Dimensions %d\n sizeToRead %d\n dataLength %d\nbytesRead %d\n Error %d 0x%8.8lx",
                              fileToLoad, imageDim[fileNumber], s, dataLength, BytesRead, e, e);
            CloseHandle( hFile ); 
            endProgress();
            return FALSE;
        }
        SendMessage(hwndPB, PBM_STEPIT, 0, 0); 
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            CloseHandle( hFile );   
            endProgress();
            sprintf(failMessage, "Escape Pressed");
            return FALSE;
        }
    }
    CloseHandle( hFile );   
    endProgress();
    
    return TRUE;
    
} // loadInputBMP



BOOL saveBMP()
{
    HANDLE             hFile;
    DWORD              BytesOut, s;
    int                i;
    HDC                hdc;
    DWORD              e;


    hdc = GetDC(mainHWND);
    dataAddress = output1;

    if (writeThru)
    {
        hFile = CreateFile(fileToSave, GENERIC_WRITE,
              FILE_SHARE_READ | FILE_SHARE_WRITE,
              NULL, CREATE_ALWAYS,
              FILE_ATTRIBUTE_NORMAL | FILE_FLAG_WRITE_THROUGH
                                    | FILE_FLAG_NO_BUFFERING, NULL);
    }
    else
    {
        hFile = CreateFile(fileToSave, GENERIC_WRITE,
              FILE_SHARE_READ | FILE_SHARE_WRITE,
              NULL, CREATE_ALWAYS,
              FILE_ATTRIBUTE_NORMAL | FILE_FLAG_NO_BUFFERING, NULL);
    }        
    if (hFile == INVALID_HANDLE_VALUE)
    {
        e = GetLastError();
        GetMemoryStatus();
        sprintf(failMessage, "Create Write Invalid Handle file %s Error %d 0x%8.8lx", fileToSave, e, e);
        ReleaseDC(mainHWND, hdc);
        return FALSE;
    }
    s = sizeToSave / filePasses[fileNumber];

    GetMemoryStatus();

    sprintf (progressText, "Writing File %s, Size %5.0f MB, Free RAM %d MB",
                        fileToSave, (double)(sizeToSave) / 1048576,
                        ramFree, pageFree);
    pbcount = filePasses[fileNumber];

    if (!startProgress())
    {
        e = GetLastError();
        GetMemoryStatus();
        CloseHandle( hFile );
        sprintf(failMessage, "startProgress failed Error %d 0x%8.8lx", e, e);
        ReleaseDC(mainHWND, hdc);
        return FALSE;
    }

    pbcount = 1; 
    SendMessage(hwndPB, PBM_SETSTEP, (WPARAM) pbcount, 0);
    
    for (i=0; i<filePasses[fileNumber]; i++)
    {
        if (WriteFile(hFile, dataAddress+s*i, s, &BytesOut, NULL) == 0)
        {
            e = GetLastError();
            GetMemoryStatus();
            sprintf(failMessage, "WriteFile %s failed, sizeToSave %d  Error %d 0x%8.8lx",
                                 fileToSave, sizeToSave, e, e);
            CloseHandle( hFile );
            endProgress();
            ReleaseDC(mainHWND, hdc);
            return FALSE;
        }
        
        if (s != BytesOut)
        {
            e = GetLastError();
            GetMemoryStatus();
            sprintf(failMessage, "WriteFile %s, sizeToSave %d, BytesOut %d  Error %d 0x%8.8lx",
                                 fileToSave, s, BytesOut, e, e);
            CloseHandle( hFile );
            endProgress();
            ReleaseDC(mainHWND, hdc);
            return FALSE;
        }
        SendMessage(hwndPB, PBM_STEPIT, 0, 0); 
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            CloseHandle( hFile );   
            endProgress();
            sprintf(failMessage, "Escape Pressed");
            ReleaseDC(mainHWND, hdc);
            return FALSE;
        }
    }
    CloseHandle( hFile );
    endProgress();   
    gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    ReleaseDC(mainHWND, hdc);
    return TRUE;

} // saveBMP

BOOL allocate1()
{
    DWORD e;
    HDC  hdc;
    
    strcpy(fileToSave, BMPfile[fileNumber]);

    imageWide = imageDim[fileNumber];
    imageHigh = imageDim[fileNumber];
    imagebpp = 24;    
    dataLength = imageWide * imageHigh * 3 + 54;
    sizeToSave = fileSize[fileNumber];

    sprintf (progressText, "Allocating %5.0f MB Memory",
                                     (double)(sizeToSave) / 1048576);
    progressHwnd = CreateDialog( MyInstance, "Progress2", mainHWND, (DLGPROC)ProgressProc2);  

    output1 = (char *)VirtualAlloc(NULL, sizeToSave, MEM_COMMIT, PAGE_READWRITE);
    if (output1 == NULL)
    {
         e = GetLastError();
         GetMemoryStatus();
         sprintf(failMessage, "VirtualAlloc 3 - Cannot allocate %d MB memory Error %d 0x%8.8lx",
                               sizeToSave / 1048576, e, e);
         if (progressHwnd)   DestroyWindow(progressHwnd);
         return FALSE;
    }
    lpOutputDIB  = (BITMAPINFO*)(output1 + sizeof(BITMAPFILEHEADER));

    bfh.bfType = 0x4d42;
    bfh.bfSize = dataLength;
    bfh.bfReserved1 = 0;
    bfh.bfReserved2 = 0;
    bfh.bfOffBits =  54;

    CopyMemory(output1, &bfh, 14);

    lpOutputDIB->bmiHeader.biSize = sizeof( BITMAPINFOHEADER );
    lpOutputDIB->bmiHeader.biWidth = imageWide;
    lpOutputDIB->bmiHeader.biHeight = imageHigh;
    lpOutputDIB->bmiHeader.biPlanes = 1;
    lpOutputDIB->bmiHeader.biBitCount = imagebpp;
    lpOutputDIB->bmiHeader.biCompression = BI_RGB;
    lpOutputDIB->bmiHeader.biSizeImage = dataLength - 54;
    lpOutputDIB->bmiHeader.biXPelsPerMeter = 0;
    lpOutputDIB->bmiHeader.biYPelsPerMeter = 0;
    lpOutputDIB->bmiHeader.biClrUsed = 0;
    lpOutputDIB->bmiHeader.biClrImportant = 0;

    if (progressHwnd)   DestroyWindow(progressHwnd);
    hdc = GetDC(mainHWND);
    gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    ReleaseDC(mainHWND, hdc);
    return TRUE;

} // allocate1    

        
BOOL enlarge()
{
    HDC  hdc;
    DWORD e;
    
    hdc = GetDC(mainHWND);


    sprintf (progressText, "Enlarge Image %d x %d to %d x %d, Size %6.1f MB",
                               imageDim[fileNumber-2], imageDim[fileNumber-2],
                               imageDim[fileNumber-2]*2, imageDim[fileNumber-2]*2,
                               (double)(sizeToSave) / 1048576);
                               
    pbcount = imageDim[fileNumber-2];

    if (!startProgress())
    {
         e = GetLastError();
         GetMemoryStatus();
         sprintf(failMessage, "startProgress failed Error %d 0x%8.8lx", e, e);
         ReleaseDC(mainHWND, hdc);
         return FALSE;
    }

    if (!enlarge2())
    {
         endProgress();
         ReleaseDC(mainHWND, hdc);
         return FALSE;
    }
    endProgress();
    gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    ReleaseDC(mainHWND, hdc);
    return TRUE;

} // enlarge

BOOL enlarge2()
{
    int i, j, c, n, p, q;
    BYTE b1, b2, b3;
    BYTE b4, b5, b6;
    BYTE b7, b8, b9;
    BYTE ba, bb, bc;
    BYTE e1, e2, e3;
    BYTE f1, f2, f3;
    char *in;
    char *out;

    in  = (char*)(inputX + 54);
    out = (char*)(output1 + 54);

    
    c = 0;
    n = 0;
    p = imageDim[fileNumber-2] * 6;
    q = imageDim[fileNumber-2] * 3;

    pbcount = 1; 
    SendMessage(hwndPB, PBM_SETSTEP, (WPARAM) pbcount, 0);

    for (i=0; i<imageDim[fileNumber-2]-1; i++)
    {
        for (j=0; j<imageDim[fileNumber-2]-1; j++)
        {
            b1 = in[c];
            b2 = in[c+1];
            b3 = in[c+2];

            e1 = in[c+3];
            e2 = in[c+4];
            e3 = in[c+5];

            f1 = in[c+q];
            f2 = in[c+1+q];
            f3 = in[c+2+q];

            b4 = b1 / 2 + e1 / 2;
            b5 = b2 / 2 + e2 / 2;
            b6 = b3 / 2 + e3 / 2;

            b7 = b1 / 2 + f1 / 2;
            b8 = b2 / 2 + f2 / 2;
            b9 = b3 / 2 + f3 / 2;

            ba = f1 / 2 + e1 / 2;
            bb = f2 / 2 + e2 / 2;
            bc = f3 / 2 + e3 / 2;

            c  = c + 3;
            out[n]     = b1;
            out[n+1]   = b2;
            out[n+2]   = b3;
            out[n+3]   = b4;
            out[n+4]   = b5;
            out[n+5]   = b6;
            out[n+p]   = b7;
            out[n+1+p] = b8;
            out[n+2+p] = b9;
            out[n+3+p] = ba;
            out[n+4+p] = bb;
            out[n+5+p] = bc;
            n = n + 6;
        }
        b4 = in[c];
        b5 = in[c+1];
        b6 = in[c+2];

        e1 = in[c-3];
        e2 = in[c-2];
        e3 = in[c-1];

        f1 = in[c+q];
        f2 = in[c+1+q];
        f3 = in[c+2+q];

        b1 = b4 / 2 + e1 / 2;
        b2 = b5 / 2 + e2 / 2;
        b3 = b6 / 2 + e3 / 2;

        ba = b4 / 2 + f1 / 2;
        bb = b5 / 2 + f2 / 2;
        bc = b6 / 2 + f3 / 2;

        b7 = f1 / 2 + e1 / 2;
        b8 = f2 / 2 + e2 / 2;
        b9 = f3 / 2 + e3 / 2;

        c  = c + 3;
        out[n]     = b1;
        out[n+1]   = b2;
        out[n+2]   = b3;
        out[n+3]   = b4;
        out[n+4]   = b5;
        out[n+5]   = b6;
        out[n+p]   = b7;
        out[n+1+p] = b8;
        out[n+2+p] = b9;
        out[n+3+p] = ba;
        out[n+4+p] = bb;
        out[n+5+p] = bc;
        n = n + p + 6;
        SendMessage(hwndPB, PBM_STEPIT, 0, 0); 
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            endProgress();
            sprintf(failMessage, "Escape Pressed");
            PostMessage(mainHWND, WM_COMMAND, MENU_EXIT, 0);
            return FALSE;
        }
    }
   
    for (j=0; j<imageDim[fileNumber-2]-1; j++)
    {
        b1 = in[c];
        b2 = in[c+1];
        b3 = in[c+2];

        e1 = in[c+3];
        e2 = in[c+4];
        e3 = in[c+5];

        f1 = in[c-q];
        f2 = in[c-q+1];
        f3 = in[c-q+2];

        b4 = b1 / 2 + e1 / 2;
        b5 = b2 / 2 + e2 / 2;
        b6 = b3 / 2 + e3 / 2;

        b7 = b1 / 2 + f1 / 2;
        b8 = b2 / 2 + f2 / 2;
        b9 = b3 / 2 + f3 / 2;

        ba = f1 / 2 + e1 / 2;
        bb = f2 / 2 + e2 / 2;
        bc = f3 / 2 + e3 / 2;

        c  = c + 3;

        out[n]     = b7;
        out[n+1]   = b8;
        out[n+2]   = b9;        
        out[n+3]   = ba;
        out[n+4]   = bb;
        out[n+5]   = bc;
        out[n+p]   = b1;
        out[n+1+p] = b2;
        out[n+2+p] = b3;
        out[n+3+p] = b4;
        out[n+4+p] = b5;
        out[n+5+p] = b6;

        n = n + 6;
        SendMessage(hwndPB, PBM_STEPIT, 0, 0); 
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            endProgress();
            sprintf(failMessage, "Escape Pressed");
            PostMessage(mainHWND, WM_COMMAND, MENU_EXIT, 0);
            return FALSE;
        }
    }

    b1 = in[c];
    b2 = in[c+1];
    b3 = in[c+2];

    out[n]     = b1;
    out[n+1]   = b2;
    out[n+2]   = b3;
    out[n+3]   = b1;
    out[n+4]   = b2;
    out[n+5]   = b3;
    out[n+p]   = b1;
    out[n+1+p] = b2;
    out[n+2+p] = b3;
    out[n+3+p] = b1;
    out[n+4+p] = b2;
    out[n+5+p] = b3;

    return TRUE;

} // enlarge2


BOOL startProgress()
{
    DWORD e;
    
    progressHwnd = CreateDialog( MyInstance, "Progress", mainHWND, (DLGPROC)ProgressProc);  

    InitCommonControls(); 
    GetClientRect(progressHwnd, &rcClient); 
    hwndPB = CreateWindowEx(0, PROGRESS_CLASS, (LPSTR) NULL, 
        WS_CHILD | WS_VISIBLE, rcClient.left+10, 
        rcClient.bottom/2, 
        rcClient.right-15, 20, 
        progressHwnd, (HMENU) 0, g_hinst, NULL);
    if (!hwndPB)
    {
        e = GetLastError();
        GetMemoryStatus();
        sprintf(failMessage, "Create Progress Window Failed Error %d 0x%8.8lx", e, e);
        return initFail();
    }
    
    // Set the range and increment of the progress bar. 
    SendMessage(hwndPB, PBM_SETRANGE, 0, MAKELPARAM(0, pbcount)); 

    return TRUE;
}

BOOL endProgress()
{
    if(hwndPB) DestroyWindow(hwndPB); 
    if (progressHwnd)   DestroyWindow(progressHwnd);
    return TRUE;
}


BOOL move1(int k, HDC hdc)
{
    int i;
    DWORD e;
    
    if (doBlt)
    {
         for (i=0; i<k; i++)
         {
             if (!BitBlt(hdc, i+2, 1, doWide, doHigh, hdcLoadX, 0, 0, SRCCOPY))
             {
                e = GetLastError();
                GetMemoryStatus();
                sprintf(failMessage, "BitBlt 3 Failed Pass %d Error %d 0x%8.8lx", passCount, e, e);
                return FALSE;
             }
         }
         if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
         {
             sprintf(failMessage, "Escape Pressed");
             return FALSE;
         }    
         for (i=k-1; i>-1; i--)
         {
             if (!BitBlt(hdc, i+2, 1, doWide, doHigh, hdcLoadX, 0, 0, SRCCOPY))
             {
                e = GetLastError();
                GetMemoryStatus();
                sprintf(failMessage, "BitBlt 4 Failed Pass %d Error %d 0x%8.8lx", passCount, e, e);
                return FALSE;
             }
         }
         if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
         {
             sprintf(failMessage, "Escape Pressed");
             return FALSE;
         }    
    }
    else
    {
        lpInputDIB1->bmiHeader.biSizeImage = imageWide * 3 * doHigh;
        lpInputDIB1->bmiHeader.biHeight = doHigh;
         for (i=0; i<k; i++)
         {
            if (StretchDIBits(hdc, i+2, 1, doWide, doHigh,
                                   0, 0, doWide,  doHigh, pBuf,
                           (LPBITMAPINFO)lpbi, DIB_RGB_COLORS, SRCCOPY) == GDI_ERROR)
             {
                e = GetLastError();
                GetMemoryStatus();
                sprintf(failMessage, "StretchDIBits 2 Failed Pass %d Error %d 0x%8.8lx", passCount, e, e);
                return FALSE;
             }
         }
         if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
         {
             sprintf(failMessage, "Escape Pressed");
             return FALSE;
         }    
         for (i=k-1; i>-1; i--)
         {
            if (StretchDIBits(hdc, i+2, 1, doWide, doHigh,
                                   0, 0, doWide,  doHigh, pBuf,
                           (LPBITMAPINFO)lpbi, DIB_RGB_COLORS, SRCCOPY) == GDI_ERROR)
             {
                e = GetLastError();
                GetMemoryStatus();
                sprintf(failMessage, "StretchDIBits 3 Failed Pass %d Error %d 0x%8.8lx", passCount, e, e);
                return FALSE;
             }
         }
         if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
         {
             sprintf(failMessage, "Escape Pressed");
             return FALSE;
         }    
    }   
    gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    return TRUE;

} // move1

BOOL move2(int k, int m, HDC hdc)
{
    int i;
    DWORD e;
    
    if (doBlt)
    {
         for (i=0; i<k; i++)
         {
             if (!BitBlt(hdc, 0, 1, doWide, doHigh, hdcLoadX, 0, i*m, SRCCOPY))
             {
                e = GetLastError();
                GetMemoryStatus();
                sprintf(failMessage, "BitBlt 5 Failed Pass %d Error %d 0x%8.8lx", passCount, e, e);
                return FALSE;
             }
         }
         if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
         {
             sprintf(failMessage, "Escape Pressed");
             return FALSE;
         }    
         for (i=k-1; i>-1; i--)
         {
             if (!BitBlt(hdc, 0, 1, doWide, doHigh, hdcLoadX, 0, i*m, SRCCOPY))
             {
                e = GetLastError();
                GetMemoryStatus();
                sprintf(failMessage, "BitBlt 6 Failed Pass %d Error %d 0x%8.8lx", passCount, e, e);
                return FALSE;
             }
         }
         if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
         {
             sprintf(failMessage, "Escape Pressed");
             return FALSE;
         }    
    }
    else
    {
         lpInputDIB1->bmiHeader.biSizeImage = imageWide * 3 * doHigh;
         lpInputDIB1->bmiHeader.biHeight = doHigh;

         for (i=0; i<k; i++) // doHigh and i*m shoud be same???
         {
            pBuf = inputX +  54  + i*m * 3 * imageWide;
            if (StretchDIBits(hdc, 0, 1, doWide, doHigh,
                                   0, 0, doWide,  doHigh, pBuf,
                           (LPBITMAPINFO)lpbi, DIB_RGB_COLORS, SRCCOPY) == GDI_ERROR)
             {
                e = GetLastError();
                GetMemoryStatus();
                sprintf(failMessage, "StretchDIBits 4 Failed Pass %d Error %d 0x%8.8lx", passCount, e, e);
                return FALSE;
             }
         }
         if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
         {
             sprintf(failMessage, "Escape Pressed");
             return FALSE;
         }    
         for (i=k-1; i>-1; i--)  
         {
            pBuf = inputX +  54  + i*m * 3 * imageWide;
            if (StretchDIBits(hdc, 0, 1, doWide, doHigh,
                                   0, 0, doWide,  doHigh, pBuf,
                           (LPBITMAPINFO)lpbi, DIB_RGB_COLORS, SRCCOPY) == GDI_ERROR)
             {
                e = GetLastError();
                GetMemoryStatus();
                sprintf(failMessage, "StretchDIBits 5 Failed Pass %d Error %d 0x%8.8lx", passCount, e, e);
                return FALSE;
             }
         }

         if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
         {
             sprintf(failMessage, "Escape Pressed");
             return FALSE;
         }    
    }   
    gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    return TRUE;

} // move2


BOOL rotate()
{
    int  i, j, c, d, e;
    int  d1;
    char *in;
    char *out;
    char buf[20];
    HDC  hdc;

        
    hdc = GetDC(mainHWND);

    in  = (char*)(inputX + 54);
    out = (char*)(output1 + 54);

    sprintf (progressText, "Rotate Image %d x %d, Size %5.0f MB",
                               imageDim[fileNumber], imageDim[fileNumber],
                               (double)(sizeToRead) / 1048576);
                               
    pbcount = imageDim[fileNumber];

    if (!startProgress())
    {
         e = GetLastError();
         GetMemoryStatus();
         sprintf(failMessage, "startProgress failed Error %d 0x%8.8lx", e, e);
         ReleaseDC(mainHWND, hdc);
         return FALSE;
    }

    pbcount = 1; 
    SendMessage(hwndPB, PBM_SETSTEP, (WPARAM) pbcount, 0);
    
    d1 = imageDim[fileNumber];
    c = 0;
    e = 3 * d1;

    for (i=0; i<d1; i++)
    {
        d = e * (d1 - 1) + i * 3;
        for (j=0; j<d1; j=j+4)
        {
            CopyMemory(buf, in+c, 12);
            CopyMemory(out+d, buf, 3);
            d = d - e;
            CopyMemory(out+d, buf+3, 3);
            d = d - e;
            CopyMemory(out+d, buf+6, 3);
            d = d - e;
            CopyMemory(out+d, buf+9, 3);
            d = d - e;
            c = c + 12;
        }
        SendMessage(hwndPB, PBM_STEPIT, 0, 0); 
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
             endProgress();
             sprintf(failMessage, "Escape Pressed");
             ReleaseDC(mainHWND, hdc);
             return FALSE;
        }
    }
    endProgress();
    gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    ReleaseDC(mainHWND, hdc);
    return TRUE;

} // rotate


displayBMP2()
{
    HDC  hdc;
    DWORD e;
        
    hdc = GetDC(mainHWND);

    sprintf (progressText, "Create BitMap & Display Image %5.0f MB", (double)(sizeToRead) / 1048576);
    progressHwnd = CreateDialog( MyInstance, "Progress2", mainHWND, (DLGPROC)ProgressProc2);  

    doBlt2 = TRUE; 
    hOutBitmap = CreateDIBitmap(hdc, (LPBITMAPINFOHEADER)lpOutputDIB,
         CBM_INIT, ((LPBITMAPINFO)lpOutputDIB)->bmiColors,
         lpOutputDIB, DIB_RGB_COLORS );
    if (hOutBitmap == NULL)
    {
        doBlt2 = FALSE;
    }
    if (doBlt2)
    {
        useBlt = useBlt + 1;    
        hdcOut = CreateCompatibleDC(hdc);
        if (hdcOut == NULL)
        {
            e = GetLastError();
            GetMemoryStatus();
            ReleaseDC(mainHWND, hdc);
            if (progressHwnd)   DestroyWindow(progressHwnd);
            sprintf(failMessage, "CreateCompatibleDC 4 Failed Error %d 0x%8.8lx", e, e);
            return FALSE;
        }
        SelectObject( hdcOut, hOutBitmap);
        
        if (!BitBlt(hdc, 0, 0, doWide, doHigh, hdcOut, 0, 0, SRCCOPY))
        {
            e = GetLastError();
            GetMemoryStatus();
            ReleaseDC(mainHWND, hdc);
            if (progressHwnd)   DestroyWindow(progressHwnd);
            sprintf(failMessage, "BitBlt R Failed Error %d 0x%8.8lx", e, e);
            return FALSE;
        }
    
    }
    else
    {
        lpOutputDIB->bmiHeader.biSizeImage = imageWide * 3 * doHigh;
        lpOutputDIB->bmiHeader.biHeight = doHigh;
       
        lpbi2 = (LPBITMAPINFOHEADER)lpOutputDIB;
        if (StretchDIBits(hdc, 0, 0, doWide, doHigh,
                                0, 0, doWide,  doHigh, output1+54,
                           (LPBITMAPINFO)lpbi2, DIB_RGB_COLORS, SRCCOPY) == GDI_ERROR)
         {
            e = GetLastError();
            GetMemoryStatus();
            ReleaseDC(mainHWND, hdc);
            if (progressHwnd)   DestroyWindow(progressHwnd);
            sprintf(failMessage, "StretchDIBits R Failed Error %d 0x%8.8lx", e, e);
            return FALSE;
         }
         lpOutputDIB->bmiHeader.biSizeImage = dataLength - 54;
         lpOutputDIB->bmiHeader.biHeight = imageHigh;
    }
    gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    GetMemoryStatus();
    if(hdcOut) DeleteDC( hdcOut );
    hdcOut = NULL;
    if(hOutBitmap) DeleteObject(hOutBitmap);
    hOutBitmap = NULL;
    if (progressHwnd)   DestroyWindow(progressHwnd);
    gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    ReleaseDC(mainHWND, hdc);
    return TRUE;

} // displayBMP2


BOOL displayBMP3(HDC hdc)
{
    DWORD e;
    
    GetClientRect(mainHWND, &clientRect);
    doWide = imageWide;
    doHigh = imageHigh;
    if (doWide > clientRect.right) doWide = clientRect.right;
    if (doHigh > clientRect.bottom) doHigh = clientRect.bottom; 
    if (doBlt2)
    {
         if (!BitBlt(hdc, 0, 0, doWide, doHigh, hdcOut, 0, 0, SRCCOPY))
         {
            e = GetLastError();
            GetMemoryStatus();
            sprintf(failMessage, "BitBlt W Failed Error %d 0x%8.8lx", e, e);
            return FALSE;
         }
         gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    }
    else
    {
        lpOutputDIB->bmiHeader.biSizeImage = imageWide * 3 * doHigh;
        lpOutputDIB->bmiHeader.biHeight = doHigh;
        if (StretchDIBits(hdc, 0, 0, doWide, doHigh,
                               0, 0, doWide,  doHigh, output1+54,
                            (LPBITMAPINFO)lpbi2, DIB_RGB_COLORS, SRCCOPY) == GDI_ERROR)

        {
                e = GetLastError();
                GetMemoryStatus();
                sprintf(failMessage, "StretchDIBits W Failed Error %d 0x%8.8lx", e, e);
                return FALSE;
        }    
        lpOutputDIB->bmiHeader.biSizeImage = dataLength - 54;
        lpOutputDIB->bmiHeader.biHeight = imageHigh;
        gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    }
    return TRUE;

} // displayBMP3   


BOOL displayBMP1(HDC hdc)
{
    DWORD e;
    
    GetClientRect(mainHWND, &clientRect);
    idown = 0;
    iright = 0;
    doWide = imageWide;
    doHigh = imageHigh;
    if (doWide > clientRect.right) doWide = clientRect.right;
    if (doHigh > clientRect.bottom) doHigh = clientRect.bottom; 
    if (imageWide > clientRect.right)
    {
        iright = (imageWide - clientRect.right) / 2;
        idown = iright; 
    }
    if (doBlt)
    {
         if (!BitBlt(hdc, 0, 0, doWide, doHigh, hdcLoadX, iright, idown, SRCCOPY))
         {
            e = GetLastError();
            GetMemoryStatus();
            sprintf(failMessage, "BitBlt 1 Failed Error %d 0x%8.8lx", e, e);
            return FALSE;
         }
    }
    else
    {
        lpInputDIBX->bmiHeader.biSizeImage = imageWide * 3 * doHigh;
        lpInputDIBX->bmiHeader.biHeight = doHigh;
        pBuf = inputX +  54  + idown * 3 * imageWide;
        if (StretchDIBits(hdc, 0, 0, doWide, doHigh,
                          iright, 0, doWide,  doHigh, pBuf,
                       (LPBITMAPINFO)lpbi, DIB_RGB_COLORS, SRCCOPY) == GDI_ERROR)

        {
                e = GetLastError();
                GetMemoryStatus();
                sprintf(failMessage, "StretchDIBits 1 Failed Error %d 0x%8.8lx", e, e);
                return FALSE;
        }
    }

    return TRUE;

} // displayBMP1   


BOOL scroll()
{
    HDC         hdc;
    BOOL        skipRun;
    int         k;
    int m = 0;
    int         kmax;
    int         mmax;
    double      passSecs;

    
    hdc = GetDC(mainHWND);

    Rectangle(hdc, clientRect.left, clientRect.top, clientRect.right, clientRect.bottom);
    start_time();

    passCount = 0;
    
    skipRun = FALSE;
    if (imageHigh-1 < clientRect.bottom)
    {
        kmax = clientRect.right - doWide -  3;
        k = 10;
        if (!move1(k, hdc))
        {
            ReleaseDC(mainHWND, hdc);
            return FALSE;
        }
        end_time();
        passSecs = secs / (double)k;
        k = (int)(((double)runTime / 2) /  passSecs); 
        if (k > kmax) k = kmax;
        if (k < 10) k = 10;
    }
    else
    {
        m = 1;
        mmax = 100;
        kmax = imageHigh - clientRect.bottom;
        
        if (kmax < 1 )
        {
            k = 1;
        }
        else if (kmax < 11)
        {
            k = kmax;
        }
        else
        {
            k = 10;
            m = kmax / k;
            if (m < 1) m = 1;
            passCount = 2 * k;
            if (!move2(k, m, hdc))
            {
                ReleaseDC(mainHWND, hdc);
                return FALSE;
            }
            end_time();
            if (secs < (double)runTime)
            {
                passSecs = secs / (double)k;
                k = (int)(((double)runTime / 2) /  passSecs); 
                if (k > kmax)
                {
                    k = kmax;
                }
                else if (k < 10)
                {
                    k = 10;
                }
                m = kmax / k;
                if (m < 1) m = 1;
            }
            else
            {
                skipRun = TRUE;
            }
        }
    }
        
    if (!skipRun)
    {
        start_time();
        passCount = 0;
        do
        {
            if (imageHigh-1 < clientRect.bottom)
            {
                if (!move1(k, hdc))
                {
                    ReleaseDC(mainHWND, hdc);
                    return FALSE;
                } 
                passCount = passCount + 2 * k;
            }
            else
            {
                if (!move2(k, m, hdc))
                {
                    ReleaseDC(mainHWND, hdc);
                    return FALSE;
                } 
                passCount = passCount + 2 * k;
            }

            gotPixel = gotPixel | GetPixel(hdc, 2, 1);
            end_time();
        }
        while (secs < (double)runTime);
    }
    ReleaseDC(mainHWND, hdc);        
    gotPixel = gotPixel | GetPixel(hdc, 2, 1);
    return TRUE;

} // scroll    


VOID GetMemoryStatus()
{
    GlobalMemoryStatusEx(&lpBuffer);

    ramMB      = (DWORD)(lpBuffer.ullTotalPhys / 1048576);
    percentMB  = lpBuffer.dwMemoryLoad;
    ramFree    = (DWORD)(lpBuffer.ullAvailPhys / 1048576);
    pageMB     = (DWORD)(lpBuffer.ullTotalPageFile / 1048576);
    pageFree   = (DWORD)(lpBuffer.ullAvailPageFile / 1048576);
    virtMB     = (lpBuffer.ullTotalVirtual / 1048576);
    virtFree   = (lpBuffer.ullAvailVirtual / 1048576);

    if (percentMB > percentUse)  percentUse  = percentMB;
    if (ramFree  < minFreeRam)  minFreeRam   = ramFree;
    if (pageFree < minFreePage) minFreePage  = pageFree;
    if (virtFree < minFreeVirt) minFreeVirt  = virtFree;
    
} // VOID GetMemoryStatus

void local_time()
{

    time_t time_of_day;
    time_of_day = time( NULL ); 
    strcpy (timeday, ctime(&time_of_day)); // timeday = date and time
    return;
}

